{
  Promathika - ein multifunktionales Programm zum Zeichnen von Funktionen

    Copyright (C) 2009  Arthur Schmidt, Alexander Staidl

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License Version 3 as published by
    the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

}

unit u_prm_numerik;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, u_prm_kernel, strutils, uGraphCom,  Dialogs;

type tGlsys =array of array of double; //Spalten x Zeilen

type
  ArgList = array of string;
  tNumerik = class
    private
      NErr:word;
      Ana: tAnalyse;
      procedure ArgumentExtrakt(Eingabe:string; var Argumente:ArgList);
      function  IstZahl(z:string):boolean;
      procedure LeerzeichenEntferner(var Eingabe:string);
      procedure SortierePunktwolke(var Punktwolke:tPointCloud);                 // Sortiert Punktwolken nach aufsteigenden x-Werten.
      function GlsysValid(Glsys:tGlsys):byte;
    public
      constructor create;
      destructor free;
      function Error:integer;
      procedure ResetError;
      procedure StrToPCloud(var Cloud:string; var resCloud:tPointCloud; var Order, Ctype: char);   // Parser; Order=s (sortiert), u (unsortiert); Ctype= n (nicht verbunden), l (linear verbunden)
      function  PCloudToStr(InputCloud:tPointCloud; Order, Ctype:char):string;                 // Generator
      function NumInt(funktion:string;dMin,dMax:extended;accuracy:integer;MathInt:boolean):extended;
      function SNumInt(funktion,dMinStr,dMaxStr,accuracyStr,MathIntStr:string):string;
      procedure SolveGLSys(var GlSys:tGlsys;var ErrorCode:byte);   //ErrorCodes: 0=alles Ok; 1=linke Seite l. abhängig, rechte nicht. 2=komplett linear abhängig; 3=Zeile=0; 4=Spalte=0
  end;
implementation


procedure tNumerik.ArgumentExtrakt(Eingabe:string; var Argumente:ArgList);
var Anz, laenge, KlAuf,n:integer;
begin
  n:=2; KlAuf:=1;                                                               // An 1. Stelle von Eingabe steht '('
  Anz:=1; setlength(Argumente,Anz);                                             // Es wird von min. 1 Argument ausgegangen
  Argumente[0]:='';
  laenge:=length(Eingabe);
  if Eingabe[n]='(' then inc(KlAuf) else if Eingabe[n]=')' then dec(KlAuf);
  repeat
    Argumente[Anz-1]:=Argumente[Anz-1] + Eingabe[n];
    inc(n);
    If (Eingabe[n]=';')and(KlAuf=1) then
    begin
      inc(Anz); setlength(Argumente,Anz);
      inc(n);
    end else if Eingabe[n]='(' then inc(KlAuf) else if Eingabe[n]=')' then dec(KlAuf);
  until (n>=laenge)or(KlAuf=0);
  If KlAuf<>0 then NErr:=5;
  If NErr=0 then for n:=0 to length(Argumente)-1 do if Argumente[n]='' then NErr:=2;
end; // ArgumentExtrakt


function tNumerik.IstZahl(z:string):boolean;
var Zb:boolean;
    n:integer;
begin
  Zb:=false;
  If not (z='-') then
  begin
    if z[1]='-' then n:=1 else n:=0;
    if z<>'' then
    repeat
      inc(n);
      if (z[n]='0')or(z[n]='1')or(z[n]='2')or(z[n]='3')or(z[n]='4')or(z[n]='5')or(z[n]='6')or(z[n]='7')
       or(z[n]='8')or(z[n]='9')or(z[n]=',')or(midstr(z,n,1)='') then Zb:=true else Zb:=false;
    until (n=length(z))or(Zb=false);
  end; // if not z='-'
  IstZahl:=Zb;
end; //IstZahl


procedure tNumerik.LeerzeichenEntferner(var Eingabe:string);
var laenge,n:dword;
begin
  n:=0; laenge:=length(Eingabe);
  repeat
    inc(n);
    if Eingabe[n]=' ' then
    begin
      delete(Eingabe,n,1);
      dec(n); dec(laenge);
    end;
  until n>=laenge;
end;

procedure tNumerik.SolveGLSys(var GlSys:tGlsys;var ErrorCode:byte);
 var
   EqSize,rows,cols,col,row,indexBiggest:dWord; //cols inkl. Ergebniss-Spalte, Rows inkl. Lösungszeile
   Progress:longInt;
   genbuf:double;
  begin
    try
    ErrorCode:=GlSysValid(GlSys);
    If (ErrorCode=0)and(length(Glsys)>1) then begin //length Glsys ist nur eine Angstabfrage, da DWord >=0
      progress:=0;
      EqSize:=length(GlSys)-1; rows:=EqSize+1; Cols:=EqSize+1;
      //Das eigentliche Lösen des GlSys beginnt hier.
      while (Progress<EqSize-1)and(ErrorCode=0) do begin
        //Sortieren der GlSys-Zeilen nach der größten progressten Variable
        IndexBiggest:=Progress; Genbuf:=0;
        //Index der Zeile mit Betragsgrößten Wert der Spalte Progress suchen.
        for row:=Progress to rows-2 do begin  //Lösungszeile wird nicht gebraucht, daher "-2"
          If abs(GenBuf)<abs(GlSys[Progress,row]) then begin
            GenBuf:=GlSys[Progress,row];
            IndexBiggest:=row;
          end;
        end;
        //Zeile Progress mit Betragsgrößten Zeile der Progressten Spalte tauschen, falls diese nicht schon ganz oben steht.
        If IndexBiggest>Progress then begin
          for Col:=0 to Cols-1 do begin
            genbuf:=GlSys[Col,Progress];
            GlSys[Col,Progress]:=GlSys[Col,IndexBiggest];
            GlSys[Col,IndexBiggest]:=genbuf;
          end;
        end;
        //Jetzt beginnt der eigentliche Rechenschritt.
        for row:= Progress+1 to rows-2 do begin
          GenBuf:=GlSys[Progress,row]/GlSys[Progress,Progress];//Quotient aus der Zeile unter Progress / Progress-Zeile an Col=Progress
          for col:=0 to Cols-1 do GlSys[Col,Row]:=GlSys[Col,Row]-genbuf*GlSys[Col,Progress];
        end;
        Inc(Progress,1);
      end;
    end;//endIf ErrorCode=0
    //Ab hier findet das Rückrechnen auf die Lösungswerte statt.
    If (ErrorCode=0)and(length(Glsys)>1)  then begin
      while (Progress>=0)and(ErrorCode=0) do begin
        Col:=Cols-2;
        genbuf:=0;
        while (Col>Progress)do begin
          genbuf:=genbuf+GlSys[Col,Rows-1]*GlSys[Col,Progress];
          dec(Col,1);
        end;
        GlSys[Progress,rows-1]:=(GlSys[Cols-1,Progress]-genbuf)/GlSys[Progress,Progress];
        dec(Progress,1);
      end;
    end;
    except
      ErrorCode:=6;
    end;
  end; //endfunction

function tNumerik.GlsysValid(Glsys:tGlsys):byte;
 var
   col,row,cols,rows:dWord;
   i:longInt;
   genbuf:double;
   comparable:boolean=false;
   RowIndependend:boolean=false;
   positivMatch:dword=1;
  begin
    cols:=length(Glsys);
    rows:=length(Glsys[0]);
    col:=0;row:=0;
    result:=0;
    i:=1;
    //showmessage(IntToStr(Cols)+'Spalten   '+IntToStr(Rows)+'Zeilen');
    while (row < rows-2)and(result=0) do begin
      //Führt den Vergleich aller Zeilen untereinander herbei
      while (result=0)and(i+row<rows-1)do begin
        //Suchen des ersten gültigen Vergleichsquotionenten, um lineare Abhängigkeit zu vergleichen
        while (col < cols)and(not comparable)and(not RowIndependend) do begin
          If (GlSys[col,row]<>0)and(GlSys[col,row+i]<>0) then begin
            genbuf:=GlSys[col,row+i]/Glsys[col,row];
            comparable:=true;
          end else if (GlSys[col,row]<>0)xor(GlSys[col,row+i]<>0) then RowIndependend:=true;
          inc(col,1);
        end;
        while (col < cols)and(comparable)and(not RowIndependend)and(positivMatch>0) do begin
          if abs(Glsys[col,row]*GenBuf-GlSys[col,row+i])<1E-10 then begin
            If (col<cols-1)then positivMatch:=2;
          end else begin
            If (col=cols-1)then result:=1;
            PositivMatch:=0; //Wenn er bei Ergebnis-Spalte hier ankommt, ist dies der erste linear unabhängige Wert. => System kaputt
          end;
          inc(col,1);
        end;
        if PositivMatch=2 then result:=2;
        comparable:=false;
        RowIndependend:=false;
        positivMatch:=1;
        Col:=0;
        inc(i,1);
      end;
      i:=1;
      Col:=0;
      comparable:=false;
      RowIndependend:=false;
      positivMatch:=1;
      inc(Row,1);
    end;
    Col:=0;Row:=0;
    //Suche, ob ganze Zeile bis min. zum Ergebnis oder eine ganze Spalte fehlt
    while (row<rows-1)and(result=0) do begin
      i:=0;
      for col:=0 to Cols-1 do begin
        If (GlSys[Col,Row]<>0)and(col<cols-1) then inc(i,1) else if (GlSys[Col,Row]<>0)and(col=cols-1)and(i=0)then i:=-1;
      end;
      if i=0 then result:=3; if i=-1 then result:=5;
      inc(row,1);
    end;
    Col:=0;Row:=0;
    while (Col<Cols-1)and(result=0) do begin
      i:=0;
      for Row:=0 to Rows-2 do begin
        If GlSys[Col,Row]<>0 then inc(i,1);
      end;
      if i=0 then result:=4;
      inc(Col,1);
    end;

  end;

procedure tNumerik.SortierePunktwolke(var Punktwolke:tPointCloud);
var n, n2, PunkteAnz: dword;
    ZwSpPunkt: tVec2D;
begin
  PunkteAnz:=length(Punktwolke);
  If PunkteAnz>1 then for n:=0 to PunkteAnz-2 do for n2:= n+1 to PunkteAnz-1 do if Punktwolke[n].x>Punktwolke[n2].x then
  begin
    ZwSpPunkt:=Punktwolke[n];
    Punktwolke[n]:=Punktwolke[n2];
    Punktwolke[n2]:=ZwSpPunkt;
  end;
end; // SortierePunktwolke


// PUBLIC

procedure tNumerik.StrToPCloud(var Cloud:string; var resCloud:tPointCloud; var Order, Ctype: char);
var Argumente: ArgList;
    n, AnzPunkte, PosTrennzeichen:dword;
    xStr,yStr:string;
begin
  NErr:=0;
  Order:='s'; Ctype:='l';
  lowercase(Cloud); LeerzeichenEntferner(Cloud);
  If Cloud[length(Cloud)-1]=';' then Nerr:=2 else ArgumentExtrakt(rightstr(Cloud,length(Cloud)-10),Argumente);     // DrawCloud
  If (NErr=0)and(length(Argumente)>=1) then
  begin
    AnzPunkte:=0;
    for n:=0 to length(Argumente)-1 do
    begin
      PosTrennzeichen:=Pos('|',Argumente[n]);
      If PosTrennZeichen>0 then
      begin
        xStr:=LeftStr(Argumente[n],PosTrennzeichen-1);
        yStr:=RightStr(Argumente[n],length(Argumente[n])-PosTrennZeichen);
        If (IstZahl(xStr))and(IstZahl(yStr)) then
        begin
          inc(AnzPunkte);
          setlength(resCloud,AnzPunkte);   // asi: Langsam, da immer wieder Länge incrementiert werden muss. Um diesen Punkt wird sich asi kümmern.
          resCloud[AnzPunkte-1].x:=StrToFloat(xStr);
          resCloud[AnzPunkte-1].y:=StrToFloat(yStr);
        end;
      end else if n=length(Argumente)-2 then
      begin
        Order:=Argumente[n][1];            // Kann Argumente[n] leer sein?
        If not (Order in ['s','u']) then NErr:=2;
      end else if n=length(Argumente)-1 then
      begin
        Ctype:=Argumente[n][1];
        If not (Ctype in ['n','l']) then NErr:=2;
      end;
    end; // for
    Cloud:=PCloudToStr(rescloud,Order,Ctype);
    If Order='s' then SortierePunktwolke(rescloud);                               // Falls Order=s wird die Punktwolke nach x sortiert.
    //Showmessage(PCloudToStr(rescloud,Order,Ctype));
  end else if NErr=0 then NErr:=8;
end; // StrToPCloud


function tNumerik.PCloudToStr(InputCloud:tPointCloud; order, Ctype:char):string;
var n: dword;
begin
  RESULT:='pointcloud(';
  for n:=0 to length(InputCloud)-1 do RESULT:=RESULT + FloatToStr(InputCloud[n].x) + '|' + FloatToStr(InputCloud[n].y) + ';';
  RESULT:=RESULT + order + ';' + Ctype + ')';
end; // PCloudToStr

function tNumerik.NumInt(funktion:string;dMin,dMax:extended;accuracy:integer;MathInt:boolean):extended;
//programmiert von Arthur Schmidt, 2006;
var
  increment,dAct,TempInt,Buffer:extended;
  i:integer;
  minusInt:boolean;
 begin
  Ana.FEing(funktion);
  minusInt:=false; TempInt:=0;
  If dMin>dMax then begin
   dAct:=dMin;
   dMin:=dMax;
   dMax:=dAct;
   minusInt:=true;
  end;
  i:=0;
  dAct:=dMin;
  increment:=(dMax-dMin)/accuracy;

  If mathInt=true then begin
   while i < accuracy do begin
    Buffer:=dAct+(increment/2);
    TempInt:=TempInt+(Ana.FWert(buffer)*increment);
    dAct:=dAct+increment;
    i:=i+1;
   end;
   //If abs(Tempint)<Power(10,-8) then Tempint:=0;
   If minusInt=true then NumInt:=TempInt*(-1) else NumInt:=TempInt;
  end
  else begin
   while i<accuracy do begin
    Buffer:=dAct+(increment/2);
    TempInt:=TempInt+(abs(Ana.FWert(buffer))*increment);
    dAct:=dAct+increment;
    i:=i+1;
   end;
   NumInt:=TempInt;
  end;
end; //NumInt


// *** PUBLIC *** PUBLIC *** PUBLIC *** PUBLIC *** PUBLIC *** PUBLIC *** PUBLIC *** PUBLIC *** PUBLIC *** PUBLIC *** PUBLIC *** PUBLIC ***

constructor tNumerik.create;
begin
  inherited create;
  Ana:=tAnalyse.create;
  NErr:=0;
  Decimalseparator:=',';
end; // create

destructor tNumerik.free;
begin
  Ana.free;
  inherited destroy;
end; // free


function tNumerik.Error:integer;
begin
  Error:=NErr;
end; // Error


procedure tNumerik.ResetError;
begin
  NErr:=0;
end; //ResetError


function tNumerik.SNumInt(funktion,dMinStr,dMaxStr,accuracyStr,MathIntStr:string):string;
var dMin,dMax:extended;
    accuracy:dword;
    MathInt:boolean;
begin
  Try
    dMin:=StrToFloat(dMinStr);
    dMax:=StrToFloat(dMaxStr);
  except
    NErr:=14; // Argument muss eine Zahl sein.
  end;
  Try
    accuracy:=StrToInt(accuracyStr);
  except
    NErr:=7; // Argument muss natürliche Zahl sein.
  end;
  If (MathIntStr='wahr')or(MathIntStr='true') then MathInt:=true else
   if (MathIntStr='falsch')or(MathIntStr='false') then MathInt:=false else NErr:=9; // unerlaubtes Zeichen verwendet.
  If NErr=0 then RESULT:=FloatToStr(NumInt(funktion,dMin,dMax,accuracy,MathInt));
  If Ana.Error<>0 then NErr:=Ana.Error;
end; // SNumInt

end.

