{
  Promathika - ein multifunktionales Programm zum Zeichnen von Funktionen

    Copyright (C) 2009  Arthur Schmidt, Alexander Staidl

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License Version 3 as published by
    the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
}


unit UDrawClass;

{$mode delphi}{$H+}

interface

uses
  Classes,SysUtils,Graphics,LResources,Forms,Controls,Dialogs,math,ExtCtrls,u_prm_numerik,uGraphCom,u_prm_settings,uFuncListDB,graphtype,intfgraphics,FpImage,lcltype;


type tFuncStats=record
  ThreadStats:Array of tThreadStats;
  PixCount:QWord;
end;

type tAllThreadresults=array of tPixArray;

type tDrawElement=record
  basefunction:tfuncset;
  extfuncs:tStrArr;
end;

type tDrawList=array of tDrawElement;

type
  tDrawClass=class
    private
      //Variablen --------------------------------------------------------------------------------------------------------------------------------------------
      ScreenBuffer,RulHor,RulVer,ScaleIco:tBitmap;
      Numerik:tNumerik;
      ClassDrwSet:tGlobalSet;
      zoomx,zoomy,IntMin,IntMax:double;             //IntMin und IntMax wird nur für Einzelfunktionen verwendet, falls erwünscht, sonst standard.
      DrawList:tDrawList;                           //Einzelfunktionen und Listen werden hier gespeichert. Bei Einzelfunktionen ist length=1
      CloudPool:tPointCloud;                        //Wenn Punktewolke
      DrawSource:byte;                              //0=Einzelgrapf, 1=Funktionsliste
      FuncStats:Array of tFuncStats;
      ClassStats:Array [0..3] of longInt;           //0=ThreadPix-Übergabe; 1=LazIntf -> bmp; 2=t_Ksystem; 3=t_frame
      //ENDE Variablen  ______________________________________________________________________________________________________________________________________
      //Funktionen  ------------------------------------------------------------------------------------------------------------------------------------------
      function XRealToCanv (const x:double):double;
      function YRealToCanv (const y:double):double;
      function dynamic_dens (xDirection:boolean):double;
      //Ende Funktionen ______________________________________________________________________________________________________________________________________
      //Proceduren  ------------------------------------------------------------------------------------------------------------------------------------------
      procedure ksystem;
      procedure PictureFrame;       //Zeichnet Umrahmung und Achsensystem-Symbol, falls eingeschaltet
      procedure DrawRuler(const horizontal:boolean;var TargetBmp:tBitmap);
      procedure DrawIco;
      Procedure DrawSingleGraph (var func:string; Settings:tGlobalSet; intmin_im,intmax_im:double; var PixThreadResults:tAllThreadResults);
      Procedure DrawFuncList;       //ruft DrawSingleGraph ein- oder mehrfach auf
      procedure DrawStats;
      procedure decodefuncString(var Term:string; var Computationmode:byte; var IntConst:double;var parastr:string);
      //Ende Prozeduren ______________________________________________________________________________________________________________________________________
    public
      //Constructor und Destructor
      constructor create;
      destructor free;
      //Setzen bzw. erneuern der Umgebungseinstellungen
      procedure SetDrawEnv2D(func:string;settings:tGlobalSet);overload;                            //Initialisierung für Einzelfunktionen, die für das ganze sichtbare Intervall gezeichnet werden sollen
      procedure SetDrawEnv2D(func:string;settings:tGlobalSet;IntMin_im,IntMax_im:double);overload; //Wie vorher, nur für ein eingeschränktes Intervall (z.B. für die Vorschaufunktion im Funktionseditor)
      procedure SetDrawEnv2D(DrawList_im:tFuncList;settings:tGlobalSet);overload;                  //Initialisierung für Funktionslisten
      //Ausgabeproceduren
      procedure Bildrueckgabe (var Ziel:tImage);                                  //Ruft alle nötigen Funktionen auf die zur Bildrückgabe führen (SetDrawEnv2D muss vorher ausgeführt worden sein)
      Procedure Linealrueckgabe (var RulHor_ref,RulVer_Ref,ScaleIco_ref:tImage);  //Falls ein Lineal gezeichnet werden soll, kann man hier die Referenzen übergeben
      Procedure PfeilOderBox(var Ziel:tImage;Start,Ende:TPoint;Pfeil:boolean);    //Zur Darstellung des Verschiebepfeils oder der Zoom-Box.
    end;

implementation
//uses umain;

  Constructor tdrawClass.create;
    begin
      inherited create;
      Screenbuffer:=tBitmap.Create; Screenbuffer.PixelFormat:=pf24bit;
      RulHor:=tBitmap.Create;
      RulVer:=tBitmap.Create;
      ScaleIco:=tBitmap.Create;
      DrawSource:=0;
      IntMin:=0; IntMax:=0;
      setlength(DrawList,0);
      Numerik:=tNumerik.Create;
    end;

//=============================================================================================================================================================

  destructor tDrawClass.free;
    begin
      Screenbuffer.Free;
      RulHor.free;
      rulVer.free;
      ScaleIco.Free;
      inherited destroy;
      DrawList:=nil;
      CloudPool:=nil;
      Numerik.free;
    end;

//=============================================================================================================================================================

  procedure tDrawClass.SetDrawEnv2D(func:string;settings:tGlobalSet);
    begin
      setlength(DrawList,1);
      DrawList[0].basefunction.FuncString:=func;
      ClassDrwSet:=Settings;
      IntMin:=0;
      IntMax:=0;
      DrawSource:=0;
      zoomx:=ClassDrwSet.MainSizing.DrawResX/(ClassDrwSet.Kartesianset.xmax-ClassDrwSet.Kartesianset.xmin);             //notwendige Werte werden initialisiert
      zoomy:=ClassDrwSet.MainSizing.DrawResY/(ClassDrwSet.Kartesianset.ymax-ClassDrwSet.Kartesianset.ymin);
      setlength(FuncStats,1);
    end;

//=============================================================================================================================================================

  procedure tDrawClass.SetDrawEnv2D(func:string;settings:tGlobalSet;IntMin_im,IntMax_im:double);
    begin
      SetDrawEnv2D(func,settings);
      IntMin:=IntMin_im;
      IntMax:=IntMax_im;
    end;

//=============================================================================================================================================================

  procedure tDrawClass.SetDrawEnv2D(DrawList_im:tFuncList;settings:tGlobalSet);
   var
     n,l:dword;
    begin
     SetDrawEnv2D('',settings);
     l:=length(DrawList_im);
     setlength(DrawList,l);
     for n:=0 to l-1 do begin
       Drawlist[n].basefunction:=DrawList_im[n];
       setlength(DrawList[n].extfuncs,0)
     end;
     DrawSource:=1;
     setlength(FuncStats,l);
    end;

//=============================================================================================================================================================

  function tDrawClass.XRealToCanv (Const x:double):double;              //Ungerundeter Wert!!!
    begin
      result:=(x-ClassDrwSet.Kartesianset.xmin)*zoomX;
    end;

//=============================================================================================================================================================

  function tDrawClass.YRealToCanv (Const Y:double):double;              //Ungerundeter Wert!!!
   begin
     result:=(ClassDrwSet.Kartesianset.yMax-Y)*zoomY;
   end;

//=============================================================================================================================================================

  function tDrawClass.dynamic_dens(xDirection:boolean):double;   //###Berechnet den optimalen Dichtewert für das Koordinatengitter
    var
      Basis,Zahlenbereich,max,min,zoom,res,grenzdichte:extended;
      Grenzwert:word;
      KorrekturFaktor:longInt;
     begin
       If xDirection then begin
         max:=ClassDrwSet.Kartesianset.xMax;
         min:=ClassDrwSet.Kartesianset.xMin;
         zoom:=zoomx;
         res:=ClassdrwSet.MainSizing.DrawResX;
       end else begin
         max:=ClassDrwSet.Kartesianset.yMax;
         min:=ClassDrwSet.Kartesianset.yMin;
         zoom:=zoomy;
         res:=ClassdrwSet.MainSizing.DrawResY;
       end;
       If res>=650 then Grenzwert:=60 else If res<=240 then Grenzwert:=10 else If (res<650)and(res>240) then Grenzwert:=19+round((res-240)*(1/10));
       Zahlenbereich:=round(ln(Max-Min)/ln(10));
       Basis:=1/Power(10,Zahlenbereich);
       Grenzdichte:=zoom/Grenzwert;
       Korrekturfaktor:=trunc(Grenzdichte/Basis);
       if KorrekturFaktor >=50 then result:=Basis*50
       else if KorrekturFaktor >=20 then result:=Basis*20
       else if KorrekturFaktor >=10 then result:=Basis*10
       else if Korrekturfaktor>=5 then result:=Basis*5
       else if Korrekturfaktor>=2 then result:=Basis*2
       else result:=Basis;
     end;

//=============================================================================================================================================================

  procedure tDrawClass.Ksystem;   //###Löscht den Screenbuffer (alle Funktionen) und zeichnet ein neues, leeres, kartesisches Koordinatensystem
   var
     xAkt,yAkt,PxCanv,PyCanv,xmin,xmax,ymin,ymax,gDichteX,gDichteY,uDichteX,uDichteY,xValueCor,yValueCor:double;
     xres,yres,rbuf,gbuf,bbuf:word;
     rechteck:trect;
     LineLength:byte;
    begin
      //---->> Lokale Variablen aus notwendigen, oft benutzten Einstellungen generieren
      xmin:=ClassDrwSet.Kartesianset.xmin;
      xmax:=ClassDrwSet.Kartesianset.xmax;
      ymin:=ClassDrwSet.Kartesianset.ymin;
      ymax:=ClassDrwSet.Kartesianset.ymax;
      xres:=ClassDrwSet.MainSizing.DrawResX;
      yres:=ClassDrwSet.MainSizing.DrawResY;
      xValueCor:=1; yValueCor:=1;
      If ClassDrwSet.Kartesianset.UseUsrDefUnits then begin
        If ClassDrwSet.Kartesianset.UnitSignX<>'' then xValueCor:=ClassDrwSet.Kartesianset.UnitValueX;
        If ClassDrwSet.Kartesianset.UnitSignY<>'' then yValueCor:=ClassDrwSet.Kartesianset.UnitValueY;
      end;

      //Blödsinnige Werte von den Dichten führen zu sehr hoher Rechenzeit! Generate Results in tGraphCom beachten und gleichermaßen abstimmen, da gleiches für GenerateResults gilt
      gDichteX:=ClassDrwSet.Kartesianset.GridDensX/xValueCor;
      If gDichteX*(xMax-xMin)>xres then GdichteX:=xres/(xMax-xMin);
      gDichtey:=ClassDrwSet.Kartesianset.GridDensY/yValueCor;
      If gDichteY*(yMax-yMin)>yres then Gdichtey:=yres/(yMax-yMin);
      uDichteX:=ClassDrwSet.Kartesianset.PinSubDensX;
      If uDichteX*gDichteX*(xMax-xMin)>xres then uDichteX:=xres/((xMax-xMin)*gDichteX);
      uDichtey:=ClassDrwSet.Kartesianset.PinSubDensY;
      If uDichteY*gDichteY*(yMax-yMin)>yres then uDichteY:=yres/((yMax-yMin)*gDichteY);

      //---->> Überzeichnen des kompletten Screenbuffers (löschen)
      With Screenbuffer do begin
        Width:=ClassDrwSet.MainSizing.DrawResX;
        Height:=ClassDrwSet.MainSizing.DrawResY;
        Canvas.brush.Color:=ClassDrwSet.ColorSet.bgrdCol;
        Rechteck.TopLeft:=Point(0,0);
        Rechteck.BottomRight:=(Point(xres-1,yres-1));
        Case ClassDrwSet.ColorSet.BgrdGrad of
          -1:Canvas.FillRect(0,0,xRes-1,yRes-1);
          0:Canvas.GradientFill(Rechteck,ClassDrwSet.ColorSet.bgrdCol,ClassDrwSet.ColorSet.bgrdCol2,gdvertical);
          1:Canvas.GradientFill(Rechteck,ClassDrwSet.ColorSet.bgrdCol,ClassDrwSet.ColorSet.bgrdCol2,gdhorizontal);
        end;
      end;

      //---->> Koordinatengitter wird nach Einstellen des Screenbuffer.Canvas.Pen berechnet und eingezeichnet
        //---> Einstellen des Pens
      With Screenbuffer.Canvas.Pen do begin
        Mode:=pmCopy;
        Width:=1;
        Color:=ClassDrwSet.ColorSet.GridCol;
      end;//endwith

        //---> Zeichnen senkrechter Koordinaten-Striche von links nach rechts, falls Dichte > 0
      If GDichteX>0 then begin
        xAkt:=(trunc(GDichteX*xmin)-1)/GDichteX;
        repeat
          PxCanv:=XRealToCanv(xAkt);
          Screenbuffer.Canvas.Polyline([point(round(PxCanv),0),point(round(PxCanv), yRes-1) ]);
          xAkt:=xAkt+(1/GdichteX);
        until PxCanv> xRes;
      end;
        //---> Zeichnen waagerechter Koordinaten-Striche von oben nach unten, falls Dichte > 0
      If GDichteY>0 then begin
        yAkt:=(trunc(GDichteY*Ymax)+1)/GDichteY;
        repeat
          PyCanv:=YRealToCanv(yAkt);
          Screenbuffer.Canvas.Polyline([Point(0,round(PyCanv)),Point(xRes,round(PyCanv)) ]);
          yAkt:=yAkt-(1/GdichteY);
        until PyCanv > yRes;
      end;
      //---->> Zeichnen der kartesichen Hauptachsen, falls sichtbar
        //---> Bestimmen der Nebenfarbe der Hauptachsen und gesamt-Breite der Hauptachsen
      if trunc(red(ClassDrwSet.ColorSet.AxisCol)+120)<256 then rbuf:=trunc(red(ClassDrwSet.ColorSet.AxisCol)+120) else rbuf:=255;
      if trunc(green(ClassDrwSet.ColorSet.AxisCol)+120)<256 then gbuf:=trunc(green(ClassDrwSet.ColorSet.AxisCol)+120) else gbuf:=255;
      if trunc(blue(ClassDrwSet.ColorSet.AxisCol)+120)<256 then bbuf:=trunc(blue(ClassDrwSet.ColorSet.AxisCol)+120) else bbuf:=255;
      Screenbuffer.Canvas.Pen.Color:=rgbToColor(rbuf,gbuf,bbuf);
      Screenbuffer.Canvas.Pen.Width:=3;
        //---> Zeichnen des breiten Y-Nebenstrichs (heller)
      If sign(xmin)<>sign(xmax) then Screenbuffer.Canvas.Polyline([Point(round(XRealToCanv(0)),0),Point(round(XRealToCanv(0)), yRes)]);
        //---> Zeichnen des breiten X-Nebenstrichs (heller)
      If sign(ymin)<>sign(ymax) then Screenbuffer.Canvas.Polyline([Point(0,round(YRealToCanv(0))),Point(xRes,round(YRealToCanv(0)))]);
        //---> Einstellungen für der Achskernbreite und Farbe
      Screenbuffer.Canvas.Pen.Color:=ClassDrwSet.ColorSet.AxisCol;
      Screenbuffer.Canvas.Pen.Width:=1;
        //---> Zeichnen der Achskerne falls sichtbar, zunächst Y-Achse, dann X-Achse
      If sign(xmin)<>sign(xmax) then Screenbuffer.Canvas.Polyline([Point(round(XRealToCanv(0)),0),Point(round(XRealToCanv(0)),yRes)]);
      If sign(ymin)<>sign(ymax) then  Screenbuffer.Canvas.Polyline([Point(0,round(YRealToCanv(0))),Point(xRes,round(YRealToCanv(0)))]);

    //---->> Skalenteilungsstriche entlang der kartesichen Hauptachsen
      //---> Festlegung der Länge der Skalenteilungsschritte
    LineLength:=1;
    If (ClassDrwSet.MainSizing.DrawResX>600) or (ClassDrwSet.MainSizing.DrawResY>450) then LineLength:=2;
    If (ClassDrwSet.MainSizing.DrawResX>900) or (ClassDrwSet.MainSizing.DrawResY>600) then LineLength:=3;
    If (ClassDrwSet.MainSizing.DrawResX>1680) or (ClassDrwSet.MainSizing.DrawResY>1200) then LineLength:=4;

      //---> Zeichnen der Skalenteilungsschritte entlang der X-Achse, wenn X-Achse sichtbar und gDichteX UND uDichteX > 0
    if (ymin<=0) and (ymax>0) and (gDichteX>0) and (uDichteX>0) then begin
      Screenbuffer.Canvas.Pen.Color:=ClassDrwSet.ColorSet.PinCol;
      xAkt:=(trunc(GDichteX*udichteX*xmin)-1)/(GDichteX*udichteX);
      PyCanv:=YrealToCanv(0);
      repeat
        PxCanv:=XRealToCanv(xAkt);
        If (abs(GDichteX*xAkt-round(xAkt*GDichteX))<1E-10) then
          Screenbuffer.Canvas.PolyLine([Point(round(PxCanv),round(PyCanv-2*linelength)),Point(round(PxCanv),round(PyCanv+2*linelength+1))])
        else Screenbuffer.Canvas.PolyLine([Point(round(PxCanv),round(PyCanv-linelength)),Point(round(PxCanv),round(PyCanv+linelength+1))]);
        xAkt:=xAkt+(1/(GdichteX*uDichteX));
      until PxCanv> xRes;
    end;
      //---> Zeichnen der Skalenteilungsschritte entlang der Y-Achse, wenn Y-Achse sichtbar und gDichteY UND uDichteY > 0
    if (xmin<=0) and (xmax>0) and (gDichteY>0) and (uDichteY>0) then begin
      Screenbuffer.Canvas.Pen.Color:=ClassDrwSet.ColorSet.PinCol;
      yAkt:=(trunc(GDichtey*uDichteY*ymax)+1)/(GDichtey*uDichteY);
      PxCanv:=xRealToCanv(0);
      repeat
        PyCanv:=yRealToCanv(yAkt);
        If (abs(GDichtey*yAkt-round(yAkt*GDichteY))<1E-10)  then
          Screenbuffer.Canvas.Polyline([Point(round(PxCanv-2*linelength),round(PyCanv)),Point(round(PxCanv+2*linelength+1),round(pyCanv))])
        else Screenbuffer.Canvas.Polyline([Point(round(PxCanv-linelength),round(Pycanv)),Point(round(PxCanv+linelength+1),round(pyCanv))]);
        yAkt:=yAkt-(1/(Gdichtey*uDichtey));
      until PyCanv> yRes;
    end;

  end; //ksystem

//=============================================================================================================================================================

  Procedure TDrawClass.PictureFrame;
   var
     xRes,yRes,i:word;
     rbuf,gbuf,bbuf:byte;
     LineLength,pxCanv,pyCanv:double;
     my_FontSize:longInt;
     StrBufx,StrBufy:String;
    begin
      if (ClassDrwSet.MainSizing.DrawResX*ClassDrwSet.MainSizing.DrawResY)>57599 then i:=2 else i:=1;  //Für Preview, also <240x240 soll kleinere Schriftgröße verwendet werden.
      my_FontSize:=i+round(SQRT(ClassDrwSet.MainSizing.DrawResX*ClassDrwSet.MainSizing.DrawResY)/1194.2*8*ClassDrwSet.FontSet.AxisFontScale); //1194,2=SQRT(1628 x 876) Entspricht Schrifthöhe 10
      //If (my_fontSize>10)and(ClassDrwSet.PerformanceSet.allowBigStatsFont=false) then my_fontSize:=10;
      Linelength:=my_FontSize/3;

      //---->> Zeichnen des Achsenkreuzes, falls eingeschalten
      If ClassDrwSet.Kartesianset.AxisSign then begin
        if trunc(red(ClassDrwSet.ColorSet.AxisCrossCol)+120)<256 then rbuf:=trunc(red(ClassDrwSet.ColorSet.AxisCrossCol)+120) else rbuf:=255;
        if trunc(green(ClassDrwSet.ColorSet.AxisCrossCol)+120)<256 then gbuf:=trunc(green(ClassDrwSet.ColorSet.AxisCrossCol)+120) else gbuf:=255;
        if trunc(blue(ClassDrwSet.ColorSet.AxisCrossCol)+120)<256 then bbuf:=trunc(blue(ClassDrwSet.ColorSet.AxisCrossCol)+120) else bbuf:=255;
        PxCanv:=Linelength*4+2;
        PyCanv:=ClassDrwSet.MainSizing.DrawResY-Linelength*4-2;
        with Screenbuffer.Canvas do begin
          Pen.Color:=ClassDrwSet.ColorSet.AxisCrossCol;
          Brush.Style:=bssolid;
          If (ClassDrwSet.ColorSet.BgrdGrad=0) then brush.Color:=ClassdrwSet.ColorSet.bgrdCol2 else brush.Color:=ClassdrwSet.ColorSet.bgrdCol;
          Polyline([Point(round(PxCanv),round(PyCanv-12*LineLength)),Point(round(PxCanv),round(PyCanv)),Point(round(PxCanv+12*LineLength),round(PyCanv))]);
          Polyline([Point(round(PxCanv-2*LineLength),round(PyCanv-9*LineLength)),Point(round(PxCanv),round(PyCanv-12*LineLength)),Point(round(PxCanv+2*LineLength),round(PyCanv-9*LineLength))]);
          Polyline([Point(round(PxCanv+9*LineLength),round(PyCanv-2*LineLength)),Point(round(PxCanv+12*LineLength),round(PyCanv)),Point(round(PxCanv+9*LineLength),round(PyCanv+2*LineLength))]);
          If (ClassDrwSet.MainSizing.DrawResX>500)and(ClassDrwSet.MainSizing.DrawResY>400)then begin
            Pen.Color:=rgbToColor(rbuf,gbuf,bbuf);
            Polyline([Point(round(PxCanv)-1,round(PyCanv-12*LineLength)+2),Point(round(PxCanv)-1,round(PyCanv)+1),Point(round(PxCanv+12*LineLength)-2,round(PyCanv)+1)]);
            Polyline([Point(round(PxCanv)+1,round(PyCanv-12*LineLength)+2),Point(round(PxCanv)+1,round(PyCanv)-1),Point(round(PxCanv+12*LineLength)-2,round(PyCanv)-1)]);
          end;
          font.Size:=my_FontSize;
          font.Color:=ClassDrwSet.ColorSet.AxisCrossCol;
          StrBufx:='x'; StrBufy:='y';
          If ClassDrwSet.Kartesianset.UseUsrDefUnits then begin
            If ClassDrwSet.Kartesianset.UnitNameX<>'' then StrBufX:=ClassDrwSet.Kartesianset.UnitNameX;
            If ClassDrwSet.Kartesianset.UnitSignX<>'' then StrBufX:=StrBufX+' ['+ClassDrwSet.Kartesianset.UnitSignX+']';
            If ClassDrwSet.Kartesianset.UnitNameY<>'' then StrBufY:=ClassDrwSet.Kartesianset.UnitNameY;
            If ClassDrwSet.Kartesianset.UnitSignY<>'' then StrBufY:=StrBufY+' ['+ClassDrwSet.Kartesianset.UnitSignY+']';
          end;
          {$ifdef linux} Fillrect(round(PxCanv+12*LineLength),round(PyCanv-7*lineLength),round(PxCanv+12*LineLength)+Screenbuffer.Canvas.TextWidth(StrBufX),round(PyCanv-7*lineLength)+Screenbuffer.Canvas.TextHeight('8')); {$endif}
          TextOut(round(PxCanv+12*LineLength),round(PyCanv-7*lineLength),StrBufX);
          font.Orientation:=900;
          {$ifdef linux} Fillrect(round(PxCanv),round(PyCanv-12*lineLength)-Screenbuffer.Canvas.TextWidth(StrBufY),round(PxCanv)+Screenbuffer.Canvas.TextHeight('8'),round(PyCanv-12*lineLength)); {$endif}
          TextOut(round(PxCanv),round(PyCanv-12*LineLength),StrBufy);
          font.Orientation:=0;
          Brush.Style:=bsSolid;
        end;
      end;

      //---->> Zeichnen der Screenbuffer-Umrahmung  (blau)
      xRes:=Screenbuffer.Width;
      yRes:=Screenbuffer.Height;

      with  Screenbuffer.Canvas do begin
        Pen.Color:=clblue;
        pen.Width:=1;
        Polyline([Point(0,0),Point(xRes-1,0),Point(xRes-1,yRes-1),Point(0,yRes-1),Point(0,0)]);
        Pen.Color:=claqua;
        Polyline([Point(1,1),Point(xRes-2,1),Point(xRes-2,yRes-2),Point(1,yRes-2),Point(1,1)]);
      end; //endwith
    end;

//=============================================================================================================================================================

  procedure tDrawClass.DrawIco;
    begin
      ScaleIco.canvas.brush.Color:=ClassDrwSet.ColorSet.RulbgrdCol; //Farbe, falls benutzerdefiniertes Lineal eingestellt
      ScaleIco.Canvas.Pen.Mode:=pmCopy;
      ScaleIco.Canvas.Pen.Width:=1;
      ScaleIco.Canvas.Brush.Style:=bssolid;

      If ClassDrwSet.Kartesianset.RulerMode=0 then begin
        If (ClassDrwSet.ColorSet.BgrdGrad=0) then ScaleIco.canvas.brush.Color:=ClassDrwSet.ColorSet.bgrdCol2
          else ScaleIco.canvas.brush.Color:=ClassDrwSet.ColorSet.bgrdCol;
      end;
      ScaleIco.Canvas.FillRect(0,0,ScaleIco.Width,ScaleIco.Height);

      with  ScaleIco.Canvas do begin
         Pen.Color:=clgreen;
         Polyline([Point(0,0),Point(ScaleIco.Width-1,0),Point(ScaleIco.Width-1,ScaleIco.Height-1),Point(0,ScaleIco.Height-1),Point(0,0)]);
         Pen.Color:=cllime;
         Polyline([Point(1,1),Point(ScaleIco.Width-2,1),Point(ScaleIco.Width-2,ScaleIco.Height-2),Point(1,ScaleIco.Height-2),Point(1,1)]);
       end;
    end;

//=============================================================================================================================================================

  procedure tDrawClass.DrawRuler(const horizontal:boolean;var TargetBmp:tBitmap); //###Löscht die Lineal-Bitmaps und Skalierungs-Bitmap und zeichnet jeweils das ganze neu
   var
    PosAct,PosCanv,GDichte,uDichte,RulDichte,min,max,zoom,ValueCor:double;
    scale,TextDistance,res,quer,my_PixFH:longint; //quer=querausdehnung des jeweiligen Lineals, my_PixFH=Sollhöhe für Schrift in Pixel
    StrBuf,UnitBuf,longestValue:String;
   begin
     If ClassDrwSet.Kartesianset.RulerMode>-1 then begin //wenn Lineal überhaupt an
       If horizontal then UnitBuf:=ClassDrwSet.Kartesianset.UnitSignX else UnitBuf:=ClassDrwSet.Kartesianset.UnitSignY;
       If (ClassDrwSet.Kartesianset.UseUsrDefUnits) and (ClassDrwSet.Kartesianset.DrwUSignRul) and (UnitBuf<>'') then begin
         if Horizontal then ValueCor:=ClassDrwSet.Kartesianset.UnitValueX else ValueCor:=ClassDrwSet.Kartesianset.UnitValueY;
         UnitBuf:='·'+UnitBuf
       end else begin
         ValueCor:=1;
         UnitBuf:='';
       end;

       //---->> Lokale Variablen aus notwendigen, oft benutzten Einstellungen generieren
       If Horizontal then begin
         RulDichte:=ClassDrwSet.Kartesianset.Ruldens_X;
         GDichte:=ClassDrwSet.Kartesianset.GridDensX;
         uDichte:=ClassDrwSet.Kartesianset.PinSubDensX;
         min:=ClassDrwSet.Kartesianset.xmin;
         max:=ClassDrwSet.Kartesianset.xmax;
         zoom:=ClassDrwSet.MainSizing.DrawResX/(max-min);
         res:=ClassDrwSet.MainSizing.DrawResX;
         quer:=Targetbmp.Height;
       end else begin
         RulDichte:=ClassDrwSet.Kartesianset.Ruldens_Y;
         GDichte:=ClassDrwSet.Kartesianset.GridDensY;
         uDichte:=ClassDrwSet.Kartesianset.PinSubDensY;
         min:=ClassDrwSet.Kartesianset.ymin;
         max:=ClassDrwSet.Kartesianset.ymax;
         zoom:=ClassDrwSet.MainSizing.DrawResY/(max-min);
         res:=ClassDrwSet.MainSizing.DrawResY;
         quer:=Targetbmp.Width;
       end;
       gDichte:=gDichte/ValueCor;
       RulDichte:=RulDichte/ValueCor;
       my_PixFH:=round(quer*0.5);
       //Schutz vor Blödsinnigen Dichtewerten, die die Rechenzeit aufblähen
       If gDichte*(Max-Min)>res then Gdichte:=res/(Max-Min);
       If uDichte*gDichte*(Max-Min)>res then uDichte:=res/((Max-Min)*gDichte);
       Scale:=round(ln(max-min)/ln(10));
       //---->> Setzen von Parametern für die TargetBmp
       TargetBmp.canvas.brush.Color:=ClassDrwSet.ColorSet.RulbgrdCol; //Farbe, falls benutzerdefiniertes Lineal
       TargetBmp.Canvas.Pen.Mode:=pmCopy;
       TargetBmp.Canvas.Pen.Width:=1;


       TargetBmp.Canvas.Brush.Style:=bssolid;

       //---->> Hintergrundgestaltung der einzelnen Images
         //---> abhängig von dem Lineal-Modus (hier benutzerdefiniert)
       If ClassDrwSet.Kartesianset.RulerMode=1 then begin
         TargetBmp.Canvas.FillRect(0,0,TargetBmp.Width,TargetBmp.Height);
       end else if ClassDrwSet.Kartesianset.RulerMode=0 then begin
         //---> ab hier automatisch angepasstes Lineal gewählt abhängig von dem Hintergrundmodus und Hintergrund-Farben
         Case ClassDrwSet.ColorSet.BgrdGrad of
           -1:begin             //kein Farbverlauf
             TargetBmp.canvas.brush.Color:=ClassDrwSet.ColorSet.bgrdCol;
             TargetBmp.Canvas.FillRect(0,0,TargetBmp.Width,TargetBmp.Height);
           end;
           0:begin              //vertikaler Farbverlauf
             If horizontal then begin
               TargetBmp.canvas.brush.Color:=ClassDrwSet.ColorSet.bgrdCol2;
               TargetBmp.Canvas.FillRect(0,0,TargetBmp.Width,TargetBmp.Height);
             end else
               TargetBmp.Canvas.GradientFill(rect(0,0,TargetBmp.Width,TargetBmp.Height),ClassDrwSet.ColorSet.bgrdCol,ClassDrwSet.ColorSet.bgrdCol2,gdvertical);
           end;
           1:begin              //horizontaler Farbverlauf
             If horizontal then TargetBmp.Canvas.GradientFill(rect(0,0,TargetBmp.Width,TargetBmp.Height),ClassDrwSet.ColorSet.bgrdCol,ClassDrwSet.ColorSet.bgrdCol2,gdhorizontal)
               else begin
                 TargetBmp.canvas.brush.Color:=ClassDrwSet.ColorSet.bgrdCol;
                 TargetBmp.Canvas.FillRect(0,0,TargetBmp.Width,TargetBmp.Height);
               end;
           end;
         end; //end case
       end; //endif rulermode=0
       //---->> Teilungsstriche für das beide Lineale, wenn GDichte und uDichte > 0
       If (gDichte>0) and (uDichte>0) then begin
         PosAct:=(trunc(GDichte*udichte*min)-1)/(GDichte*udichte);
         If ClassDrwSet.Kartesianset.RulerMode=0 then TargetBmp.Canvas.Pen.Color:=ClassDrwSet.ColorSet.PinCol      //abhängige Farbe der Teilstriche
           else TargetBmp.Canvas.Pen.Color:=ClassDrwSet.ColorSet.RulTextCol;
         repeat
           If horizontal then PosCanv:=XrealToCanv(PosAct) else PosCanv:=YRealToCanv(PosAct);
           If (abs(GDichte*posAct-round(posAct*GDichte))<1E-10)then begin
             If horizontal then TargetBmp.Canvas.PolyLine([Point(round(PosCanv),round(0.15*quer)),Point(round(PosCanv),round(0.4*quer))])
             else TargetBmp.Canvas.PolyLine([Point(round(quer*0.6),round(PosCanv)),Point(round(quer*0.85),round(PosCanv))]);
           end else begin
             If horizontal then TargetBmp.Canvas.PolyLine([Point(round(PosCanv),round(0.22*quer)),Point(round(PosCanv),round(0.36*quer))]) //Linien auf ganzen Zahlen sind länger
             else TargetBmp.Canvas.PolyLine([Point(round(quer*0.64),round(PosCanv)),Point(round(quer*0.78),round(PosCanv))]); //Linien auf ganzen Zahlen sind länger
           end;
           posAct:=posAct+(1/(Gdichte*uDichte));
         until ((horizontal)and(PosCanv>TargetBmp.Width))or((not horizontal)and(PosCanv<0));
       end;

       //---->> Beschriftungen der TargetBmp, wenn Beschriftungsdichte > 0
       If RulDichte>0 then begin
         LongestValue:='';
         PosAct:=(trunc(RulDichte*min)-1)/RulDichte;
         If horizontal then TargetBmp.Canvas.Font.Orientation:=0 else TargetBmp.Canvas.Font.Orientation:=900;
         TextDistance:=round(((1/RulDichte))*zoom);
           //-> Festlegung von Brush-Style und Schriftgröße
         TargetBmp.Canvas.Brush.Style:=bsclear;
         TargetBmp.Canvas.Font.height:=round(my_PixFH*1.5);
         Targetbmp.Canvas.Font.Name:='Arial';
         //Um zu ermitteln, wie groß die Schrift ausfallen kann, muss der Pixel-längste String gefunden werden
         repeat
           StrBuf:=FloatToStr(round(power(10,-Scale+3)*posAct/ValueCor)/power(10,-Scale+3))+UnitBuf;
           If TargetBmp.Canvas.TextWidth(StrBuf)>TargetBmp.Canvas.TextWidth(LongestValue) then longestValue:=StrBuf;
           PosAct:=PosAct+(1/RulDichte);
         until ((horizontal)and(PosCanv>TargetBmp.Width))or((not horizontal)and(PosCanv<0));
         PosAct:=(trunc(RulDichte*min)-1)/RulDichte;
         //Wenn jetzt die Beschritung zu breit (>TextDistance) oder die Texthöhe zu hoch (je nach Widgetset und DPI), wird diese reduziert, bis es passt.
         while ((TargetBmp.Canvas.TextHeight('8') > my_PixFH)or(TargetBmp.Canvas.TextWidth(LongestValue+'8')>TextDistance))and (TargetBmp.Canvas.TextHeight('8') > 5) do begin
           TargetBmp.Canvas.Font.height:=TargetBmp.Canvas.Font.height-1;
         end;

         //Wenn die reale Texthöhe noch so, dass man was lesen kann, wird eine Beschriftung ausgegeben, sonst eine Info zur Senkung der Beschriftungsdichte.
         If (TargetBmp.Canvas.TextHeight('8') > 5)or(TargetBmp.Height*TargetBmp.Width<57601) then begin
           //-> Festlegung der Schriftfarbe abhängig vom Linealmodus
           If ClassDrwSet.Kartesianset.RulerMode=0 then TargetBmp.Canvas.Font.Color:=ClassDrwSet.ColorSet.PinCol
             else TargetBmp.Canvas.Font.Color:=ClassDrwSet.ColorSet.RulTextCol;
           repeat
             if horizontal then PosCanv:=XrealToCanv(PosAct) else PosCanv:=YRealToCanv(PosAct);
             StrBuf:=FloatToStr(round(power(10,-Scale+3)*posAct/ValueCor)/power(10,-Scale+3))+UnitBuf;
             if horizontal then TargetBmp.Canvas.TextOut(round(PosCanv)-(TargetBmp.Canvas.TextWidth(strBuf)div 2),round(quer*0.4),StrBuf) //Beschriftung
             else TargetBmp.Canvas.TextOut(round(quer*0.1),round(PosCanv)+(TargetBmp.Canvas.TextWidth(strBuf)div 2),StrBuf);
             PosAct:=PosAct+(1/RulDichte);
           until ((horizontal)and(PosCanv>TargetBmp.Width))or((not horizontal)and(PosCanv<0));
         end;
       end; //endIf RulDichte >0
       //---->> Umrandungen der Targetbmp
       with  TargetBmp.Canvas do begin
         Pen.Color:=clgreen;
         pen.Width:=1;
         Polyline([Point(0,0),Point(TargetBmp.Width-1,0),Point(TargetBmp.Width-1,TargetBmp.Height-1),Point(0,TargetBmp.Height-1),Point(0,0)]);
         Pen.Color:=cllime;
         Polyline([Point(1,1),Point(TargetBmp.Width-2,1),Point(TargetBmp.Width-2,TargetBmp.Height-2),Point(1,TargetBmp.Height-2),Point(1,1)]);
       end;
     end;//endif rulermode>-1: Hauptschleife, falls Lineal aktiviert
  end; //end procedure

//=============================================================================================================================================================

  procedure tDrawClass.DrawStats;
   var
     my_fontSize,LH,StatsCount:longInt;
     TextOffsetX,TextOffsetY,i,n,k,pFTC:longInt;   //pFTC=perFuncThreadCount; n=Zeilenzähler
     strBuf,RowBuf:String;
     loc_ThreadStats:array [0..1]of  tThreadStats; //0=min-Time; 1=max_Time
     start:tDateTime;
     stpcount:int64;
    begin
      start:=time;
      stpcount:=0;
      if ClassDrwSet.PerformanceSet.statistics then n:=7 else n:=0;
      if (ClassDrwSet.MainSizing.DrawResX*ClassDrwSet.MainSizing.DrawResY)>57599 then i:=2 else i:=1;  //Für Preview, also <240x240 soll kleinere Schriftgröße verwendet werden.
      my_fontSize:=i+round(SQRT(ClassDrwSet.MainSizing.DrawResX*ClassDrwSet.MainSizing.DrawResY)/1194.2*8*ClassDrwSet.FontSet.StatsFontScale); //1194,2=SQRT(1628 x 876) Entspricht Schrifthöhe 10
      //If (my_fontSize>10)and(ClassDrwSet.PerformanceSet.allowBigStatsFont=false) then my_fontSize:=10;
      LH:=round(My_FontSize*1.7);
      with Screenbuffer.Canvas do begin
        Brush.Style:=bssolid;
        Pen.Color:=ClassDrwSet.ColorSet.bgrdCol;
        Brush.Color:=ClassDrwSet.ColorSet.bgrdCol;
        Font.size:=my_fontSize;
      end;

      StatsCount:=length(FuncStats);

      TextOffsetX:=trunc(ClassDrwSet.PerformanceSet.TermkoordX*ClassDrwSet.MainSizing.DrawResX*0.01);
      TextOffsetY:=trunc(ClassDrwSet.PerformanceSet.TermkoordY*ClassDrwSet.MainSizing.DrawResY*0.01);


      If (ClassDrwSet.PerformanceSet.TermOut>-1)or(DrawSource=0) then begin
        for i:=0 to StatsCount-1 do begin
          stpCount:=0;
          If ((drawSource=1)and(DrawList[i].basefunction.Active))or(DrawSource=0) then begin
            If DrawSource=1 then Screenbuffer.Canvas.Font.Color:=DrawList[i].basefunction.CurvCol else ScreenBuffer.Canvas.Font.Color:=ClassDrwSet.ColorSet.CurvCol;
            StrBuf:=IntToStr(i)+': ';
            if DrawSource=1 then begin
              Case ClassDrwSet.PerformanceSet.TermOut of
                0: StrBuf:=StrBuf+'f(x)='+DrawList[i].basefunction.FuncString;
                1: StrBuf:=StrBuf+DrawList[i].basefunction.FuncName;
                2: if DrawList[i].basefunction.FuncName<>'' then StrBuf:=StrBuf+DrawList[i].basefunction.FuncName+' => '+'f(x)='+DrawList[i].basefunction.FuncString else StrBuf:=StrBuf+'f(x)='+DrawList[i].basefunction.FuncString;
              end;
            end else StrBuf:=StrBuf+'f(x)='+Drawlist[0].basefunction.FuncString;
            pFTC:=length(FuncStats[i].ThreadStats)-1;
            //Initialwerte für die Stats setzen

            loc_ThreadStats[0].Resettime:=2147483647; loc_ThreadStats[1].Resettime:=0;
            loc_ThreadStats[0].RenderTime:=2147483647; loc_ThreadStats[1].RenderTime:=0;
            loc_ThreadStats[0].postprocess:=2147483647; loc_ThreadStats[1].postprocess:=0;
            loc_ThreadStats[0].parser:=2147483647; loc_ThreadStats[1].parser:=0;
            //Min- und max-Werte herausfiltern
            for k:=0 to pFTC do begin
              If FuncStats[i].ThreadStats[k].Resettime<loc_ThreadStats[0].Resettime then loc_ThreadStats[0].Resettime:=FuncStats[i].ThreadStats[k].Resettime;
              If FuncStats[i].ThreadStats[k].Resettime>loc_ThreadStats[1].Resettime then loc_ThreadStats[1].Resettime:=FuncStats[i].ThreadStats[k].Resettime;
              If FuncStats[i].ThreadStats[k].Rendertime<loc_ThreadStats[0].Rendertime then loc_ThreadStats[0].Rendertime:=FuncStats[i].ThreadStats[k].Rendertime;
              If FuncStats[i].ThreadStats[k].Rendertime>loc_ThreadStats[1].Rendertime then loc_ThreadStats[1].Rendertime:=FuncStats[i].ThreadStats[k].Rendertime;
              If FuncStats[i].ThreadStats[k].Postprocess<loc_ThreadStats[0].Postprocess then loc_ThreadStats[0].Postprocess:=FuncStats[i].ThreadStats[k].Postprocess;
              If FuncStats[i].ThreadStats[k].Postprocess>loc_ThreadStats[1].Postprocess then loc_ThreadStats[1].Postprocess:=FuncStats[i].ThreadStats[k].Postprocess;
              If FuncStats[i].ThreadStats[k].parser<loc_ThreadStats[0].parser then loc_ThreadStats[0].parser:=FuncStats[i].ThreadStats[k].parser;
              If FuncStats[i].ThreadStats[k].parser>loc_ThreadStats[1].parser then loc_ThreadStats[1].parser:=FuncStats[i].ThreadStats[k].parser;
              StpCount:=StpCount+FuncStats[i].ThreadStats[k].StepCount;
            end;

            RowBuf:= StrBuf+' | t_init:('+IntToStr(loc_ThreadStats[0].resettime)+'..'+IntToStr(loc_ThreadStats[1].resettime)+')ms'+
                     ' | t_render:('+IntToStr(loc_ThreadStats[0].rendertime)+'..'+IntToStr(loc_ThreadStats[1].rendertime)+')ms'+
                     ' | t_postfx:('+IntToStr(loc_ThreadStats[0].postprocess)+'..'+IntToStr(loc_ThreadStats[1].postprocess)+')ms | '+ IntToStr(FuncStats[i].PixCount)+'Pix'+
                     ' | t_parser:('+IntToStr(loc_ThreadStats[0].parser)+'..'+IntToStr(loc_ThreadStats[1].parser)+')ms'+' | Werte: '+IntToStr(StpCount);

            {$ifdef linux}
              if ClassDrwSet.PerformanceSet.statistics then
                Screenbuffer.Canvas.Fillrect(TextOffsetX,n*LH+TextOffsetY,TextOffsetX+2+Screenbuffer.Canvas.TextWidth(Rowbuf),n*LH+TextOffsetY+Screenbuffer.Canvas.TextHeight('8'))
              else Screenbuffer.Canvas.Fillrect(TextOffsetX,n*LH+TextOffsetY,TextOffsetX+2+Screenbuffer.Canvas.TextWidth(StrBuf),n*LH+TextOffsetY+Screenbuffer.Canvas.TextHeight('8'));
            {$endif}
            if ClassDrwSet.PerformanceSet.statistics then Screenbuffer.Canvas.TextOut(TextOffsetX,n*LH+TextOffsetY,RowBuf)
              else Screenbuffer.Canvas.TextOut(TextOffsetX,n*LH+TextOffsetY,StrBuf);
            inc(n,1);
          end;
        end;
      end;//wenn einfaches TermOut erwünscht

      If ClassDrwSet.PerformanceSet.statistics then begin
        n:=0;
        with Screenbuffer.Canvas do begin
          Font.Color:=ClassdrwSet.ColorSet.GridCol;

          RowBuf:='Zeichenfläche: '+IntToStr(ClassDrwSet.MainSizing.DrawResX)+' x '+IntToStr(ClassDrwSet.MainSizing.DrawResY)+' Pixel';
          {$ifdef linux} Fillrect(TextOffsetX,n*LH+TextOffsetY,TextOffsetX+Screenbuffer.Canvas.TextWidth(RowBuf),n*LH+TextOffsetY+Screenbuffer.Canvas.TextHeight('8')); {$endif}
          TextOut(TextOffsetX,n*LH+TextOffsetY,Rowbuf); inc(n,1);

          RowBuf:='t_Ksystem: '+IntToStr(ClassStats[2])+' ms';
          {$ifdef linux} Fillrect(TextOffsetX,n*LH+TextOffsetY,TextOffsetX+Screenbuffer.Canvas.TextWidth(RowBuf),n*LH+TextOffsetY+Screenbuffer.Canvas.TextHeight('8'));; {$endif}
          TextOut(TextOffsetX,n*LH+TextOffsetY,Rowbuf); inc(n,1);

          RowBuf:='t_Receive_Results: '+IntToStr(ClassStats[0])+' ms';
          {$ifdef linux} Fillrect(TextOffsetX,n*LH+TextOffsetY,TextOffsetX+Screenbuffer.Canvas.TextWidth(RowBuf),n*LH+TextOffsetY+Screenbuffer.Canvas.TextHeight('8'));; {$endif}
          TextOut(TextOffsetX,n*LH+TextOffsetY,Rowbuf); inc(n,1);

          RowBuf:='t_convert_Results: '+IntToStr(ClassStats[1])+' ms';
          {$ifdef linux} Fillrect(TextOffsetX,n*LH+TextOffsetY,TextOffsetX+Screenbuffer.Canvas.TextWidth(RowBuf),n*LH+TextOffsetY+Screenbuffer.Canvas.TextHeight('8')); {$endif}
          TextOut(TextOffsetX,n*LH+TextOffsetY,Rowbuf); inc(n,1);

          RowBuf:='t_frame: '+IntToStr(ClassStats[3])+' ms';
          {$ifdef linux} Fillrect(TextOffsetX,n*LH+TextOffsetY,TextOffsetX+Screenbuffer.Canvas.TextWidth(RowBuf),n*LH+TextOffsetY+Screenbuffer.Canvas.TextHeight('8')); {$endif}
          TextOut(TextOffsetX,n*LH+TextOffsetY,Rowbuf); inc(n,1);

          RowBuf:='t_DrawStats: '+IntToStr(round((Time-start)*24*60*60*1000))+' ms';
          {$ifdef linux} Fillrect(TextOffsetX,n*LH+TextOffsetY,TextOffsetX+Screenbuffer.Canvas.TextWidth(RowBuf),n*LH+TextOffsetY+Screenbuffer.Canvas.TextHeight('8')); {$endif}
          TextOut(TextOffsetX,n*LH+TextOffsetY,Rowbuf); inc(n,1);
        end;
      end;
    end;

//=============================================================================================================================================================

  procedure tDrawClass.decodefuncString(var Term:string; var Computationmode:byte; var IntConst:double; var parastr:string);
  var
     w,posc,poss:dword; //posc=Position von ')' ; poss=Position von ';'
     strbuf:string;
     order,ctype:char;
   begin
     parastr:='';
     IntConst:=0;
     If Pos('drwint(',LowerCase(Term))<>0 then Computationmode:=1 else if Pos('pointcloud(',LowerCase(Term))<>0 then Computationmode:=2 else Computationmode:=0;
     If Computationmode=1 then  begin
       strbuf:=trim(lowercase(Term));
       strbuf:=Copy(Strbuf,8,length(Strbuf)-8);
       posc:=0;
       poss:=0;
       w:=length(strbuf);
       repeat
         If Copy(StrBuf,w,1)=')' then posc:=w;
         If Copy(StrBuf,w,1)=';' then poss:=w;
         dec(w,1);
       until (posc>0)or(w<1);
       If (Poss=0) then begin  //Sicher ohne Parameter
         term:=StrBuf;
       end else begin
         Term:=Copy(StrBuf,1,Poss-1);
         try
           IntConst:=StrToFloat(Copy(StrBuf,poss+1,length(Strbuf)-poss));
         except
           //IntConst ist ja bereits 0 von anfang an, falls except eintritt
         end;
       end;
     end else if Computationmode=2 then begin
       Numerik.StrToPCloud(Term,CloudPool,order,ctype);
       parastr:=Order+ctype;
       numerik.ResetError;
     end;
  end;

//=============================================================================================================================================================

  Procedure tDrawClass.DrawSingleGraph(var func:string; Settings:tGlobalSet; intmin_im,intmax_im:double; var PixThreadResults:tAllThreadResults);
   var
     Workers:array[0..31]of tGraphCom;
     i,steps:integer;
     t_Buffer:tDateTime;
     IntLimit,CurIntOffset,Intmin_loc,IntMax_loc:double;
     StrBuf,para:string;
     compmode:byte;  //0=normal; 1=NumInt; 2=PointCloud
    begin
     setlength(PixThreadResults, Settings.PerformanceSet.ThreadCount);
     setlength(FuncStats[length(FuncStats)-1].ThreadStats, Settings.PerformanceSet.ThreadCount);
     If IntMin_im<Intmax_im then begin  //Wenn Zeichenmodus = numerisches Interal, dann braucht man nachfolgende Zeilen zur Bestimmung der unteren
       IntMin_loc:=IntMin_im;        //Integrationsgrenze.
       IntMax_loc:=IntMax_im;
     end else begin
       IntMin_loc:=Settings.Kartesianset.xmin;
       IntMax_loc:=Settings.Kartesianset.xmax;
     end;
     strbuf:=func;
     Decodefuncstring(Strbuf,compmode,CurIntOffset,para);
     If Compmode=1 then begin
       Steps:=trunc((IntMax_Loc-IntMin_loc)/(Settings.PerformanceSet.unsharpnes/zoomx));
       IntLimit:=Numerik.Numint(StrBuf,IntMin_loc,0,Steps,true);
     end;
     If CompMode=2 then begin
       steps:=length(CloudPool);
       If (Copy(para,1,1)='u') or (Copy(para,2,1)='n') or (steps<2) then Settings.PerformanceSet.AA_Mode:=3
       else if (Copy(para,1,1)='s') and ((Copy(para,2,1)='l')) then Settings.PerformanceSet.AA_Mode:=4;
     end;
     for i:=0 to Settings.PerformanceSet.ThreadCount-1 do begin
       If (Compmode=0) or (Compmode=1)then Workers[i]:=tGraphCom.create(true,Settings,i,IntMin_im,IntMax_im,StrBuf);
       If Compmode=1 then begin
         If i>0 then CurIntOffset:=CurIntOffset+Numerik.NumInt(Strbuf,IntMin_loc+((i-1)*(Intmax_loc-Intmin_loc)/Settings.PerformanceSet.ThreadCount),IntMin_loc+(i*(Intmax_loc-Intmin_loc)/Settings.PerformanceSet.ThreadCount),Steps div Settings.PerformanceSet.ThreadCount+1,true);
         Workers[i].SetIntegralOffset(CurIntOffset-IntLimit);
       end;
       If Compmode=2 then begin
         Workers[i]:=tGraphCom.create(true,Settings,i,IntMin_im,IntMax_im,CloudPool);
       end;
       Workers[i].Resume;
     end;
     //hier müsste man eine "Initialisierung Aussen" festhalten FuncStats round((Time-t_Buffer)*24*60*60*1000)

     for i:=0 to Settings.PerformanceSet.ThreadCount-1 do begin
       Workers[i].WaitFor;
     end;

     FuncStats[length(FuncStats)-1].PixCount:=0;
     For i:=0 to Settings.PerformanceSet.ThreadCount-1 do begin
       PixThreadResults[i]:=Workers[i].GetPixArray;
       FuncStats[length(FuncStats)-1].ThreadStats[i]:=Workers[i].GetTimes;
       FuncStats[length(FuncStats)-1].PixCount:=FuncStats[length(FuncStats)-1].PixCount+length(PixThreadResults[i]);
     end;

     For i:=0 to Settings.PerformanceSet.ThreadCount-1 do begin
       workers[i].Free;
     end;
    end;

//=============================================================================================================================================================

  Procedure tDrawClass.DrawFuncList;
   var
     i,k,funcs:longInt;
     ErgArray:tAllThreadResults;
     BufSet:tGlobalSet;
     ScrMaskH,ScrBufH:HBitmap;
     ColBuf:tColor;
     loc_pixbuf:tPix;
     IntfImg:tLazIntfImage;
     t_buffer,t_buf_ThreadPix:tDateTime;
    begin
      BufSet:=ClassDrwSet;
      ScrBufH:=Screenbuffer.Handle;
      ScrMaskH:=Screenbuffer.MaskHandle;
      IntfImg:=TLazIntfImage.Create(0, 0);
      IntfImg.LoadFromBitmap(ScrBufH,ScrMaskH);
      t_Buf_ThreadPix:=0;
      If DrawSource=1 then begin
        setlength(FuncStats,0);
        For funcs:=0 to length(DrawList)-1 do begin
          //--->> Wenn Funktion aktiv, wird zumindest die Statistik ausgegeben, selbst wenn Funktion aus dem sichtbaren Bereich verschoben
          setlength(FuncStats,length(funcstats)+1);
          If (DrawList[funcs].basefunction.Active)then begin

            BufSet.ColorSet.CurvCol:=DrawList[funcs].basefunction.CurvCol;
            BufSet.ColorSet.CurvCol2:=DrawList[funcs].basefunction.CurvCol2;
            BufSet.ColorSet.CurvGrad:=DrawList[funcs].basefunction.CurvGrad;
            BufSet.PerformanceSet.AA_Level_hor:=DrawList[funcs].basefunction.AA_Level_hor;
            BufSet.PerformanceSet.AA_Level_ver:=DrawList[funcs].basefunction.AA_Level_ver;
            BufSet.PerformanceSet.AA_Mode:=DrawList[funcs].basefunction.AA_Mode;
            BufSet.PerformanceSet.CurvWidth:=DrawList[funcs].basefunction.CurvWidth;
            Bufset.PerformanceSet.unsharpnes:=DrawList[funcs].basefunction.unsharpnes;
            BufSet.PerformanceSet.ThreadCount:=DrawList[funcs].basefunction.ThreadCount;

            DrawSingleGraph(DrawList[Funcs].basefunction.FuncString, BufSet, DrawList[funcs].basefunction.IntMin, DrawList[funcs].basefunction.IntMax, ErgArray);
            t_Buffer:=time;
            for i:=0 to BufSet.PerformanceSet.ThreadCount-1 do begin
              for k:=0 to length(ErgArray[i])-1 do begin
                loc_pixbuf:=ErgArray[i,k];
                ColBuf:=loc_pixbuf.Farbe;
                IntfImg.Colors[loc_PixBuf.Punkt.x, loc_Pixbuf.Punkt.y] := FPColor(red(ColBuf)*256,green(ColBuf)*256,blue(Colbuf)*256);
              end;
            end;
            t_Buf_ThreadPix:=t_Buf_ThreadPix+time-t_Buffer;
          end;
        end;
      end else begin
        setlength(FuncStats,1);
        DrawSingleGraph(Drawlist[0].basefunction.FuncString, ClassDrwSet, IntMin, IntMax, ErgArray);
        t_Buffer:=time;
        for i:=0 to BufSet.PerformanceSet.ThreadCount-1 do begin
          for k:=0 to length(ErgArray[i])-1 do begin
            loc_pixbuf:=ErgArray[i,k];
            ColBuf:=loc_pixbuf.Farbe;
            IntfImg.Colors[loc_pixbuf.Punkt.x, loc_pixbuf.Punkt.y] := FPColor(red(ColBuf)*256,green(ColBuf)*256,blue(Colbuf)*256);
          end;
        end;
        t_Buf_ThreadPix:=t_Buf_ThreadPix+time-t_Buffer;
      end;
      ClassStats[0]:=round((t_Buf_ThreadPix)*24*60*60*1000);
      t_Buffer:=time;
      IntfImg.CreateBitmaps(ScrBufH,ScrMaskH);
      Screenbuffer.Handle:=ScrBufH;
      Screenbuffer.MaskHandle:=ScrMaskH;
      IntfImg.free;
      ClassStats[1]:=round((time-t_buffer)*24*60*60*1000);
    end;

//=============================================================================================================================================================

  procedure tDrawClass.Bildrueckgabe(var Ziel:tImage);
    var
      t_Buffer:tDateTime;
    begin
      If ClassDrwSet.Kartesianset.dynamicGrid then begin
        ClassDrwSet.Kartesianset.GridDensX:= dynamic_dens(true);
        ClassDrwSet.Kartesianset.GridDensY:= dynamic_dens(false);
        ClassDrwSet.Kartesianset.Ruldens_X:=ClassDrwSet.Kartesianset.GridDensX;
        ClassDrwSet.Kartesianset.Ruldens_Y:=ClassDrwSet.Kartesianset.GridDensY;
      end;
      t_Buffer:=time;
      Ksystem;
      ClassStats[2]:=round((Time-t_Buffer)*24*60*60*1000);
      DrawFuncList;
      t_Buffer:=time;
      PictureFrame;
      ClassStats[3]:=round((Time-t_Buffer)*24*60*60*1000);
      If (ClassDrwSet.PerformanceSet.statistics) or (ClassDrwSet.PerformanceSet.TermOut>-1) then DrawStats;
      Ziel.Canvas.Draw(0,0,ScreenBuffer);
    end;

//=============================================================================================================================================================

  Procedure tDrawClass.Linealrueckgabe (var RulHor_ref,RulVer_Ref,ScaleIco_ref:tImage);
    begin

      RulHor.Width:=RulHor_ref.Width;
      RulHor.Height:=RulHor_ref.Height;
      RulVer.Width:=RulVer_ref.Width;
      RulVer.Height:=RulVer_ref.Height;
      ScaleIco.Width:=RulHor.Height;
      ScaleIco.Height:=ScaleIco.Width;

      DrawRuler(true,RulHor);
      DrawRuler(false,RulVer);
      DrawIco;
      RulHor_ref.Canvas.Draw(0,0,RulHor);
      RulVer_ref.Canvas.Draw(0,0,RulVer);
      ScaleIco_ref.Canvas.Draw(0,0,ScaleIco);
    end;

//=============================================================================================================================================================

  Procedure tDrawClass.PfeilOderBox(var Ziel:tImage;Start,Ende:TPoint;Pfeil:boolean);

  var
    tx,ty,u,v,phi:real;
  begin
    with Ziel.Canvas.Pen do begin
      Mode:=pmNotXor;
      Color:=clBlack;//ClassDrwSet.ColorSet.PinCol;
      Width:=1;
    end;
    If Pfeil then begin
      with Ziel.Canvas do begin
        MoveTo(round(start.x),round(start.Y));
        LineTo(round(ende.x),round(ende.Y));
      end; //endwith
      If (Ende.X<>Start.X) or (ende.Y<>start.Y) then begin                               //Winkelberechnung fr die Koordinatentransformation
       Phi:=(sign(ende.y-start.y))*arccos((ende.x-start.x)/sqrt(sqr(ende.x-start.x)+sqr(ende.y-start.y)));
       if(Phi=0)and (Ende.X<Start.X) then Phi:=-Pi; //an dieser Stelle wrde Phi nach -Pi springen mssen, tuts aber nicht, deshalb
       u:=sqrt(sqr(ende.x-start.x)+sqr(ende.y-start.y))-16;
       v:=5;                                                      //Darstellen und Berechnen des einen Pfeilspitzenastes
       tx:=(u*cos(phi))-(v*tan(phi)*cos(phi));
       ty:=((u-v*tan(phi))*sin(phi))+(v/cos(phi));
       Ziel.Canvas.MoveTo(round(start.x+tx),round(start.y+ty));
       Ziel.Canvas.LineTo(round( ende.x ),round( ende.y ));
       v:=-5;                                                     //Darstellen und Berechnen des anderen Pfeilspitzenastes
       tx:=(u*cos(phi))-(v*tan(phi)*cos(phi));
       ty:=((u-v*tan(phi))*sin(phi))+(v/cos(phi));
       Ziel.Canvas.MoveTo(round(start.x+tx),round(start.y+ty));
       Ziel.Canvas.LineTo(round(ende.x),round(ende.y));
      end;
     end;//end if Pfeil

     If not Pfeil then begin
       If (Ende.X<>Start.X) and (ende.Y<>start.Y) then begin
          with Ziel.Canvas do begin
            Polyline([Start,point(Ende.X,Start.Y),Ende,point(Start.x,Ende.y),Start]);
          end;//endwith
       end;
     end;//end If not Pfeil
  end;


end.

