{
  Promathika - ein multifunktionales Programm zum Zeichnen von Funktionen

    Copyright (C) 2009  Arthur Schmidt, Alexander Staidl

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License Version 3 as published by
    the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
}


unit UKonsole;

{$mode objfpc}{$H+}

interface

uses Sysutils, u_prm_kernel, u_prm_numerik, lclintf, dialogs;                                   // dialogs dient nur zum Debuggen

type
  ArgList = array of string;
  tKonsole = class
    private
      KErr:word;
      DezimalbruchZiffernAnz:word;                                                     // 0: Exakte Ausgabe mit Brüchen, Konstanten, u.s.w.; 1-19: Ausgabe der Zahlen auf 1-19 Ziffern genau.
      CAS:tAnalyse;
      Numerik:tNumerik;
      procedure LeerzeichenEntferner(var Eingabe:string);                              // Entfernt alle Leerzeichen auf Eingabe
      function TermeRek(Eingabe:string):string;                                        // Terme-Rekursion - rekursive Konsole für Termoperationen
      procedure ArgumentExtrakt(Eingabe:string; var Argumente:ArgList; var n:integer); // Extrahiert die Argumente einer Funktion aus einem String. n=ZeichenAnzahl
      function SelbstTest(TextEing:string; Modus:word):string;                         // Führt einen Selbsttest der Konsole durch.
      function SetzeDezBruchZiffAnz(Eingabe:string):string;
    public
      constructor create;
      destructor free;

      function BIOS(Eingabe:string):string;
      function ErrorCode:word;
      function ErrorMsg:string;
  end;



implementation

procedure tKonsole.LeerzeichenEntferner(var Eingabe:string);
var laenge,n:dword;
begin
  n:=0; laenge:=length(Eingabe);
  repeat
    inc(n);
    if Eingabe[n]=' ' then
    begin
      delete(Eingabe,n,1);
      dec(n); dec(laenge);
    end;
  until n>=laenge;
end;


function tKonsole.TermeRek(Eingabe:string):string;
var Position, n, ZeichenAnz:integer;
    Argumente:ArgList;
    AbbruchBed, NichtVereinfachen:boolean;
begin
  AbbruchBed:=false;
  NichtVereinfachen:=false;                                                                   // Hintergrund: Im Fall von numerischen Ergebnissen ist eine Vereinfachung nicht erwünscht, da diese ansonsten in rationale Zahlen umgewandelt werden.

  //Befehl: SelbstTest
  Position:=Pos('selbsttest(',Eingabe);
  If Position<>0 then
  begin
    ArgumentExtrakt(rightstr(Eingabe,length(Eingabe)-Position+1-10),Argumente,ZeichenAnz);
    If (Kerr=0)and(length(Argumente)=2) then
    begin
      If length(Argumente[1])=1 then if (ord(Argumente[1][1])>48)and(ord(Argumente[1][1])<54) then
      begin
        delete(Eingabe,Position,ZeichenAnz+10);
        insert(Selbsttest(Argumente[0],StrToInt(Argumente[1])),Eingabe,Position);
      end;
      AbbruchBed:=true;
    end else if KErr=0 then KErr:=8;
    NichtVereinfachen := true;
  end else
  while not AbbruchBed do                                                                    // Hier beginnt die eigentliche Befehlsprüfung.
  begin
    AbbruchBed:=true;
    //Befehl: Basiswechsel
    Position:=Pos('basiswechsel(',Eingabe);
    If Position<>0 then
    begin
      ArgumentExtrakt(rightstr(Eingabe,length(Eingabe)-Position+1-12),Argumente,ZeichenAnz);
      If length(Argumente)=3 then                                                             // Besonderheit Basiswechsel: Der ZahlString darf nicht nochmal rekursiv durchleuchtet werden, da er das sonst illegale |-Zeichen enthalten kann.
      begin
        Argumente[0]:=TermeRek(Argumente[0]);
        Argumente[1]:=TermeRek(Argumente[1]);
      end;
      If (Kerr=0)and(length(Argumente)=3) then
      begin
        delete(Eingabe,Position,ZeichenAnz+12);
        insert(CAS.SBasiswechsel(Argumente[0],Argumente[1],Argumente[2]),Eingabe,Position);
        If CAS.Error<>0 then KErr:=CAS.Error else
        begin
          AbbruchBed:=false;
          NichtVereinfachen:=true;
        end;
      end else if KErr=0 then KErr:=8;
    end; // Basiswechsel

    //Befehl: dezimalbruch
    Position:=Pos('dezimalbruch(',Eingabe);
    If Position<>0 then
    begin
      ArgumentExtrakt(rightstr(Eingabe,length(Eingabe)-Position+1-12),Argumente,ZeichenAnz);   // -12, da 'entw' nicht mitübergeben werden soll.
      //For n:=0 to length(Argumente)-1 do Argumente[n]:=TermeRek(Argumente[n]);
      If (Kerr=0)and(length(Argumente)=1) then
      begin
        delete(Eingabe,Position,ZeichenAnz+12);                                                // +12, da 'entw' mitgezählt werden muss.
        insert('(' + SetzeDezBruchZiffAnz(Argumente[0]) + ')',Eingabe,Position);
        If CAS.Error<>0 then KErr:=CAS.Error else AbbruchBed:=false;
      end else if KErr=0 then KErr:=8;
    end; // dezimalbruch

    // Befehl: diff
    Position:=Pos('diff(',Eingabe);
    If Position<>0 then
    begin
      ArgumentExtrakt(rightstr(Eingabe,length(Eingabe)-Position+1-4),Argumente,ZeichenAnz);   // -4, da 'diff' nicht mitübergeben werden soll.
      For n:=0 to length(Argumente)-1 do Argumente[n]:=TermeRek(Argumente[n]);
      If (Kerr=0)and(length(Argumente)=2) then
      begin
        delete(Eingabe,Position,ZeichenAnz+4);                                                // +4, da 'diff' mitgezählt werden muss.
        insert('(' + CAS.SAbleitung(Argumente[0],Argumente[1]) + ')',Eingabe,Position);
        If CAS.Error<>0 then KErr:=CAS.Error else AbbruchBed:=false;
      end else if (Kerr=0)and(length(Argumente)=3) then
      begin
        delete(Eingabe,Position,ZeichenAnz+4);                                                // +4, da 'diff' mitgezählt werden muss.
        insert('(' + CAS.SAbleitung(Argumente[0],Argumente[1],Argumente[2]) + ')',Eingabe,Position);
        If CAS.Error<>0 then KErr:=CAS.Error else AbbruchBed:=false;
      end else if KErr=0 then KErr:=8;
    end; // diff

    //Befehl: div
    Position:=Pos('div(',Eingabe);
    If Position<>0 then
    begin
      ArgumentExtrakt(rightstr(Eingabe,length(Eingabe)-Position+1-3),Argumente,ZeichenAnz);
      For n:=0 to length(Argumente)-1 do Argumente[n]:=TermeRek(Argumente[n]);
      If (Kerr=0)and(length(Argumente)=2) then
      begin
        delete(Eingabe,Position,ZeichenAnz+3);
        insert('(' + CAS.Sdiv(Argumente[0],Argumente[1]) + ')',Eingabe,Position);
        If CAS.Error<>0 then KErr:=CAS.Error else AbbruchBed:=false;
      end else if KErr=0 then KErr:=8;
    end; // div

    // Befehl: entw
    Position:=Pos('entw(',Eingabe);
    If Position<>0 then
    begin
      ArgumentExtrakt(rightstr(Eingabe,length(Eingabe)-Position+1-4),Argumente,ZeichenAnz);   // -4, da 'entw' nicht mitübergeben werden soll.
      For n:=0 to length(Argumente)-1 do Argumente[n]:=TermeRek(Argumente[n]);
      If (Kerr=0)and(length(Argumente)=1) then
      begin
        delete(Eingabe,Position,ZeichenAnz+4);                                                // +4, da 'entw' mitgezählt werden muss.
        insert(CAS.SEntwicklung(Argumente[0]),Eingabe,Position);
        If CAS.Error<>0 then KErr:=CAS.Error else AbbruchBed:=false;
        NichtVereinfachen:=true;
      end else if KErr=0 then KErr:=8;
    end; // entw

    //Befehl: Fakultaet
    Position:=Pos('fakultaet(',Eingabe);
    If Position<>0 then
    begin
      ArgumentExtrakt(rightstr(Eingabe,length(Eingabe)-Position+1-9),Argumente,ZeichenAnz);
      For n:=0 to length(Argumente)-1 do Argumente[n]:=TermeRek(Argumente[n]);
      If (Kerr=0)and(length(Argumente)=1) then
      begin
        delete(Eingabe,Position,ZeichenAnz+9);
        insert('(' + CAS.SFakultaet(Argumente[0]) + ')',Eingabe,Position);
        If CAS.Error<>0 then KErr:=CAS.Error else AbbruchBed:=false;
      end else if KErr=0 then KErr:=8;
    end; // Fakultaet

    //Befehl: ggt
    Position:=Pos('ggt(',Eingabe);
    If Position<>0 then
    begin
      ArgumentExtrakt(rightstr(Eingabe,length(Eingabe)-Position+1-3),Argumente,ZeichenAnz);
      For n:=0 to length(Argumente)-1 do Argumente[n]:=TermeRek(Argumente[n]);
      If (Kerr=0)and(length(Argumente)=2) then
      begin
        delete(Eingabe,Position,ZeichenAnz+3);
        insert('(' + CAS.Sggt(Argumente[0],Argumente[1]) + ')',Eingabe,Position);
        If CAS.Error<>0 then KErr:=CAS.Error else AbbruchBed:=false;
      end else if KErr=0 then KErr:=8;
    end; // ggt

    //Befehl: istprim
    Position:=Pos('istprim(',Eingabe);
    If Position<>0 then
    begin
      ArgumentExtrakt(rightstr(Eingabe,length(Eingabe)-Position+1-7),Argumente,ZeichenAnz);
      For n:=0 to length(Argumente)-1 do Argumente[n]:=TermeRek(Argumente[n]);
      If (Kerr=0)and(length(Argumente)=1) then
      begin
        delete(Eingabe,Position,ZeichenAnz+7);
        insert('(' + CAS.SIstPrim(Argumente[0]) + ')',Eingabe,Position);
        If CAS.Error<>0 then KErr:=CAS.Error else AbbruchBed:=false;
      end else if KErr=0 then KErr:=8;
    end; // istprim

    //Befehl: kgv
    Position:=Pos('kgv(',Eingabe);
    If Position<>0 then
    begin
      ArgumentExtrakt(rightstr(Eingabe,length(Eingabe)-Position+1-3),Argumente,ZeichenAnz);
      For n:=0 to length(Argumente)-1 do Argumente[n]:=TermeRek(Argumente[n]);
      If (Kerr=0)and(length(Argumente)=2) then
      begin
        delete(Eingabe,Position,ZeichenAnz+3);
        insert('(' + CAS.Skgv(Argumente[0],Argumente[1]) + ')',Eingabe,Position);
        If CAS.Error<>0 then KErr:=CAS.Error else AbbruchBed:=false;
      end else if KErr=0 then KErr:=8;
    end; // kgv

    // Befehl: Loese
    Position:=Pos('loese(',Eingabe);
    If Position<>0 then
    begin
      ArgumentExtrakt(rightstr(Eingabe,length(Eingabe)-Position+1-5),Argumente,ZeichenAnz);
      //For n:=0 to length(Argumente)-1 do Argumente[n]:=TermeRek(Argumente[n]);       <<--- muss später dennoch ermöglicht werden.
      If (Kerr=0)and(length(Argumente)=2) then
      begin
        delete(Eingabe,Position,ZeichenAnz+5);
        insert(CAS.SLoeseGleichung(Argumente[0],Argumente[1]),Eingabe,Position);
        If CAS.Error<>0 then KErr:=CAS.Error else
        begin
          AbbruchBed:=false;
          NichtVereinfachen:=true;
        end;
      end else if KErr=0 then KErr:=8;
    end; // Loese

    //Befehl: MillerRabinItera
    Position:=Pos('millerrabinitera(',Eingabe);
    If Position<>0 then
    begin
      ArgumentExtrakt(rightstr(Eingabe,length(Eingabe)-Position+1-16),Argumente,ZeichenAnz);
      For n:=0 to length(Argumente)-1 do Argumente[n]:=TermeRek(Argumente[n]);
      If (Kerr=0)and(length(Argumente)=2) then
      begin
        delete(Eingabe,Position,ZeichenAnz+16);
        insert('(' + CAS.SmillerrabinPrim(Argumente[0],Argumente[1]) + ')',Eingabe,Position);
        If CAS.Error<>0 then KErr:=CAS.Error else AbbruchBed:=false;
      end else if KErr=0 then KErr:=8;
    end; // MillerRabinItera

    //Befehl: MillerRabinPrim
    Position:=Pos('millerrabinprim(',Eingabe);
    If Position<>0 then
    begin
      ArgumentExtrakt(rightstr(Eingabe,length(Eingabe)-Position+1-15),Argumente,ZeichenAnz);
      For n:=0 to length(Argumente)-1 do Argumente[n]:=TermeRek(Argumente[n]);
      If (Kerr=0)and(length(Argumente)=2) then
      begin
        delete(Eingabe,Position,ZeichenAnz+15);
        insert('(' + CAS.SmillerrabinPrim(Argumente[0],Argumente[1]) + ')',Eingabe,Position);
        If CAS.Error<>0 then KErr:=CAS.Error else AbbruchBed:=false;
      end else if KErr=0 then KErr:=8;
    end; // MillerRabinPrim

    //Befehl: mod
    Position:=Pos('mod(',Eingabe);
    If Position<>0 then
    begin
      ArgumentExtrakt(rightstr(Eingabe,length(Eingabe)-Position+1-3),Argumente,ZeichenAnz);
      For n:=0 to length(Argumente)-1 do Argumente[n]:=TermeRek(Argumente[n]);
      If (Kerr=0)and(length(Argumente)=2) then
      begin
        delete(Eingabe,Position,ZeichenAnz+3);
        insert('(' + CAS.Smod(Argumente[0],Argumente[1]) + ')',Eingabe,Position);
        If CAS.Error<>0 then KErr:=CAS.Error else AbbruchBed:=false;
      end else if KErr=0 then KErr:=8;
    end; // mod

    //Befehl: NueberK
    Position:=Pos('nueberk(',Eingabe);
    If Position<>0 then
    begin
      ArgumentExtrakt(rightstr(Eingabe,length(Eingabe)-Position+1-7),Argumente,ZeichenAnz);
      For n:=0 to length(Argumente)-1 do Argumente[n]:=TermeRek(Argumente[n]);
      If (Kerr=0)and(length(Argumente)=2) then
      begin
        delete(Eingabe,Position,ZeichenAnz+7);
        insert('(' + CAS.SNueberK(Argumente[0],Argumente[1]) + ')',Eingabe,Position);
        If CAS.Error<>0 then KErr:=CAS.Error else AbbruchBed:=false;
      end else if KErr=0 then KErr:=8;
    end; // NueberK

    //Befehl: numint
    Position:=Pos('numint(',Eingabe);
    If Position<>0 then
    begin
      ArgumentExtrakt(rightstr(Eingabe,length(Eingabe)-Position+1-6),Argumente,ZeichenAnz);
      For n:=0 to length(Argumente)-1 do if (n<>1) and (n<>2) then Argumente[n]:=TermeRek(Argumente[n]);
      If (Kerr=0)and(length(Argumente)=4) then
      begin
        delete(Eingabe,Position,ZeichenAnz+6);
        insert(Numerik.SNumInt(Argumente[0],Argumente[1],Argumente[2],Argumente[3],'wahr'),Eingabe,Position);
        If Numerik.Error<>0 then KErr:=Numerik.Error else
        begin
          AbbruchBed:=false;
          NichtVereinfachen:=true;
        end;
      end else if (Kerr=0)and(length(Argumente)=5) then
      begin
        delete(Eingabe,Position,ZeichenAnz+6);
        insert(Numerik.SNumInt(Argumente[0],Argumente[1],Argumente[2],Argumente[3],Argumente[4]),Eingabe,Position);
        If Numerik.Error<>0 then KErr:=Numerik.Error else
        begin
          AbbruchBed:=false;
          NichtVereinfachen:=true;
        end;
      end else if KErr=0 then KErr:=8;
    end; // numint

    //Befehl: PrimDanach
    Position:=Pos('primdanach(',Eingabe);
    If Position<>0 then
    begin
      ArgumentExtrakt(rightstr(Eingabe,length(Eingabe)-Position+1-10),Argumente,ZeichenAnz);
      For n:=0 to length(Argumente)-1 do Argumente[n]:=TermeRek(Argumente[n]);
      If (Kerr=0)and(length(Argumente)=1) then
      begin
        delete(Eingabe,Position,ZeichenAnz+10);
        insert('(' + CAS.SPrimDanach(Argumente[0]) + ')',Eingabe,Position);
        If CAS.Error<>0 then KErr:=CAS.Error else AbbruchBed:=false;
      end else if KErr=0 then KErr:=8;
    end; // primdanach

    //Befehl: PrimDavor
    Position:=Pos('primdavor(',Eingabe);
    If Position<>0 then
    begin
      ArgumentExtrakt(rightstr(Eingabe,length(Eingabe)-Position+1-9),Argumente,ZeichenAnz);
      For n:=0 to length(Argumente)-1 do Argumente[n]:=TermeRek(Argumente[n]);
      If (Kerr=0)and(length(Argumente)=1) then
      begin
        delete(Eingabe,Position,ZeichenAnz+9);
        insert('(' + CAS.SPrimDavor(Argumente[0]) + ')',Eingabe,Position);
        If CAS.Error<>0 then KErr:=CAS.Error else AbbruchBed:=false;
      end else if KErr=0 then KErr:=8;
    end; // primdavor

    //Befehl: PrimFaktorZerlegung
    Position:=Pos('primfaktorzerlegung(',Eingabe);
    If Position<>0 then
    begin
      ArgumentExtrakt(rightstr(Eingabe,length(Eingabe)-Position+1-19),Argumente,ZeichenAnz);
      For n:=0 to length(Argumente)-1 do Argumente[n]:=TermeRek(Argumente[n]);
      If (Kerr=0)and(length(Argumente)=1) then
      begin
        delete(Eingabe,Position,ZeichenAnz+19);
        insert(CAS.SPrimFaktZerl(Argumente[0]),Eingabe,Position);
        If CAS.Error<>0 then KErr:=CAS.Error else AbbruchBed:=false;
        NichtVereinfachen:=true;
      end else if KErr=0 then KErr:=8;
    end; // PrimFaktorZerlegung

    //Befehl: PrimzahlListe
    Position:=Pos('primzahlliste(',Eingabe);
    If Position<>0 then
    begin
      ArgumentExtrakt(rightstr(Eingabe,length(Eingabe)-Position+1-13),Argumente,ZeichenAnz);
      For n:=0 to length(Argumente)-1 do Argumente[n]:=TermeRek(Argumente[n]);
      If (Kerr=0)and(length(Argumente)=2) then
      begin
        delete(Eingabe,Position,ZeichenAnz+13);
        insert(CAS.SPrimzahlListe(Argumente[0],Argumente[1]),Eingabe,Position);
        If CAS.Error<>0 then KErr:=CAS.Error else AbbruchBed:=false;
        NichtVereinfachen:=true;
      end else if KErr=0 then KErr:=8;
    end; // PrimzahlListe

    //Befehl: PotenzModulo  HINWEIS: Dieser Befehl soll später vom Vereinfachungsalgorithmus automatisch bei Modulo-Operationen im Fall von a^b mod n ausgeführt werden !!!!!
    Position:=Pos('potenzmodulo(',Eingabe);
    If Position<>0 then
    begin
      ArgumentExtrakt(rightstr(Eingabe,length(Eingabe)-Position+1-12),Argumente,ZeichenAnz);
      For n:=0 to length(Argumente)-1 do Argumente[n]:=TermeRek(Argumente[n]);
      If (Kerr=0)and(length(Argumente)=3) then
      begin
        delete(Eingabe,Position,ZeichenAnz+12);
        insert('(' + CAS.SPotenzModulo(Argumente[0],Argumente[1],Argumente[2]) + ')',Eingabe,Position);
        If CAS.Error<>0 then KErr:=CAS.Error else AbbruchBed:=false;
      end else if KErr=0 then KErr:=8;
    end; // PotenzModulo

    // Befehl: Taylor
    Position:=Pos('taylor(',Eingabe);
    If Position<>0 then
    begin
      ArgumentExtrakt(rightstr(Eingabe,length(Eingabe)-Position+1-6),Argumente,ZeichenAnz);   // -4, da 'entw' nicht mitübergeben werden soll.
      For n:=0 to length(Argumente)-1 do Argumente[n]:=TermeRek(Argumente[n]);
      If (Kerr=0)and(length(Argumente)=4) then
      begin
        delete(Eingabe,Position,ZeichenAnz+6);                                                // +4, da 'entw' mitgezählt werden muss.
        insert(CAS.STaylorEntwicklung(Argumente[0], Argumente[1], Argumente[2], Argumente[3]),Eingabe,Position);
        If CAS.Error<>0 then KErr:=CAS.Error else AbbruchBed:=false;
        NichtVereinfachen:=true;
      end else if KErr=0 then KErr:=8;
    end; // Taylor

    //Befehl: zufallszahl
    Position:=Pos('zufallszahl(',Eingabe);
    If Position<>0 then
    begin
      ArgumentExtrakt(rightstr(Eingabe,length(Eingabe)-Position+1-11),Argumente,ZeichenAnz);
      For n:=0 to length(Argumente)-1 do Argumente[n]:=TermeRek(Argumente[n]);
      If (Kerr=0)and(length(Argumente)=1) then
      begin
        delete(Eingabe,Position,ZeichenAnz+11);
        insert('(' + CAS.SGNZRandom(Argumente[0]) + ')',Eingabe,Position);
        If CAS.Error<>0 then KErr:=CAS.Error else AbbruchBed:=false;
      end else if KErr=0 then KErr:=8;
    end; // zufallszahl

  end; // while not AbbruchBed
  If Kerr<>0 then RESULT:='' else if NichtVereinfachen then RESULT:=Eingabe else           // Im Fall von numerischen Ergebnissen ist eine Vereinfachung nicht erwünscht, da diese ansonsten in rationale Zahlen umgewandelt werden.
  begin
    RESULT:=CAS.SVereinfachung(Eingabe, DezimalbruchZiffernAnz);
    Kerr:=CAS.Error;
    If Kerr<>0 then RESULT:='';
  end;
end; // TermeRek


procedure tKonsole.ArgumentExtrakt(Eingabe:string; var Argumente:ArgList; var n:integer);
var Anz, laenge, KlAuf:integer;
begin
  n:=2; KlAuf:=1;                                                               // An 1. Stelle von Eingabe steht '('
  Anz:=1; setlength(Argumente,Anz);                                             // Es wird von min. 1 Argument ausgegangen
  Argumente[0]:='';
  laenge:=length(Eingabe);
  if Eingabe[n]='(' then inc(KlAuf) else if Eingabe[n]=')' then dec(KlAuf);
  repeat
    Argumente[Anz-1]:=Argumente[Anz-1] + Eingabe[n];
    inc(n);
    If (Eingabe[n]=';')and(KlAuf=1) then
    begin
      inc(Anz); setlength(Argumente,Anz);
      inc(n);
    end else if Eingabe[n]='(' then inc(KlAuf) else if Eingabe[n]=')' then dec(KlAuf);
  until (n>=laenge)or(KlAuf=0);
  If KlAuf<>0 then KErr:=5;
end; // ArgumentExtrakt


function tKonsole.SelbstTest(TextEing:string; Modus:word):string;
var Datei: TextFile;
    Eingabe, ErwAusgabe: array of string;
    ZwSp: string;
    n, TrennZeichen, Anzahl, Startzeit: dword;
    DateiExistiert, Uebereinstimmung:boolean;
begin                                                                          // Schaltet Fehlerüberprüfung ein.
  Startzeit:=gettickcount;
  DateiExistiert:=FileExists('Vergleichsdaten.pdd');
  If DateiExistiert then
  begin
    n:=0; setlength(Eingabe,1); setlength(ErwAusgabe,1);
    AssignFile(Datei,'Vergleichsdaten.pdd');                                      // pdd = Promathika Debugging Datei
    Reset(Datei);
    while not EoF(Datei) do
    begin
      inc(n);
      setlength(Eingabe,n); setlength(ErwAusgabe,n);
      Readln(Datei, ZwSp);
      TrennZeichen:=Pos('#',ZwSp);                                              // Später um weitere Argumente durch hinzufügen weiterer #-Zeichen ausbaubar.
      Eingabe[n-1]:=copy(ZwSp,1,TrennZeichen-1);
      ErwAusgabe[n-1]:=copy(ZwSp,TrennZeichen+1,length(ZwSp));
    end; //while
  end;
  CASE Modus of
  1:  If DateiExistiert then
      begin                                                                   // 1 = Vergleiche ALLE Vergleichsdaten
        n:=length(Eingabe);
        repeat
          dec(n);
          Uebereinstimmung := BIOS(Eingabe[n])=ErwAusgabe[n];
        until (n=0)or(not Uebereinstimmung);
        If Uebereinstimmung then RESULT:='Konsolenausgabe stimmt mit allen Vergleichsdaten überein.'
         else RESULT:='Eingabe: ' + Eingabe[n] + '; Erwartete Ausgabe: ' + ErwAusgabe[n] + '; Ausgabe: ' + BIOS(Eingabe[n]);
      end; // 1 = Vergleiche ALLE Vergleichsdaten
  2:  If DateiExistiert then
      begin                                                                   // 2 = Vergleicht alle Vergleichsdaten, die den Textabschnitt TextEing enthalten
        n:=length(Eingabe);
        Uebereinstimmung:=true;
        repeat
          dec(n);
          If Pos(TextEing,Eingabe[n])<>0 then Uebereinstimmung := BIOS(Eingabe[n])=ErwAusgabe[n];
        until (n=0)or(not Uebereinstimmung);
        If Uebereinstimmung then RESULT:='Konsolenausgabe stimmt mit allen Vergleichsdaten, die den Text "' + TextEing + '" enthalten überein.'
         else RESULT:='Eingabe: ' + Eingabe[n] + '; Erwartete Ausgabe: ' + ErwAusgabe[n] + '; Ausgabe: ' + BIOS(Eingabe[n]);
      end; // 2 = Vergleiche Auswahl
  3:  If DateiExistiert then
      begin                                                                   // 3 = Alle Vergleichsdaten werden durch neue Konsolenausgaben überschrieben
        for n:= 0 to length(Eingabe)-1 do ErwAusgabe[n]:=BIOS(Eingabe[n]);
        RESULT:='Die Vergleichsdaten wurden überschrieben';
      end; // 3 = Vergleichsdaten überschreiben
  4:  If DateiExistiert then
      begin                                                                   // 4 = Alle Vergleichsdaten, die den Textabschnitt TextEing enthalten, werden durch neue Konsolenausgaben überschrieben
        RESULT:='';
        for n:= 0 to length(Eingabe)-1 do If Pos(TextEing,Eingabe[n])<>0 then
        begin
          ErwAusgabe[n]:=BIOS(Eingabe[n]);
          RESULT:='Die ausgewählten Vergleichsdaten wurden überschrieben';
        end;
        If RESULT='' then RESULT:='Keine Vergleichsdaten wurden überschrieben.';
      end; // 4 = Ausgewählte Vergleichsdaten überschreiben
  5:  begin                                                                   // 5 = Ein Vergleichsdatum hinzufügen
        n:=length(Eingabe);
        setlength(Eingabe,n+1); setlength(ErwAusgabe,n+1);
        Eingabe[n]:=TextEing;
        ErwAusgabe[n]:=BIOS(TextEing);
        RESULT:=ErwAusgabe[n];
      end;
  end; // CASE
  If DateiExistiert then CloseFile(Datei);
  If Modus>=3 then
  begin
    try
      AssignFile(Datei,'Vergleichsdaten.pdd');
      Rewrite(Datei);
      for n:=0 to length(Eingabe)-1 do writeln(Datei,Eingabe[n]+'#'+ErwAusgabe[n]);
    finally
      CloseFile(Datei);
    end;
  end;
  RESULT:=RESULT + '   ::   Bearbeitungszeit: ' + IntToStr(gettickcount-Startzeit) + ' ms';
end; // SelbstTest


function tKonsole.SetzeDezBruchZiffAnz(Eingabe:string):string;
var ZiffAnz:word;
    n:dword;
begin
  ZiffAnz:=DezimalbruchZiffernAnz;
  for n:=1 to length(Eingabe) do if (ord(Eingabe[n])<48)or(ord(Eingabe[n])>57) then KErr:=9;
  If KErr=0 then ZiffAnz:=StrToInt(Eingabe);
  If n>=16 then KErr:=18;
  If KErr=0 then DezimalbruchZiffernAnz:=ZiffAnz;
  RESULT:=IntToStr(DezimalbruchZiffernAnz);
end;


// *** PUBLIC *** PUBLIC *** PUBLIC *** PUBLIC *** PUBLIC *** PUBLIC *** PUBLIC *** PUBLIC *** PUBLIC *** PUBLIC *** PUBLIC *** PUBLIC ***

constructor tKonsole.create;
begin
  inherited create;
  CAS:=tAnalyse.create;
  Numerik:=tNumerik.create;
  KErr:=0;
  DezimalbruchZiffernAnz:=0;
end; // create

destructor tKonsole.free;
begin
  CAS.free;
  Numerik.free;
  inherited destroy;
end; // free


function tKonsole.BIOS(Eingabe:string):string;
begin
  CAS.ResetError; Numerik.ResetError; KErr:=0;
  If length(Eingabe)>0 then
  begin
    lowercase(Eingabe);
    LeerzeichenEntferner(Eingabe);
    RESULT:=TermeRek(Eingabe);
  end else KErr:=14;
  If KErr=0 then KErr:=CAS.Error;
  If KErr<>0 then RESULT:='ERROR: ' + IntToStr(KErr) + ' ' + ErrorMsg;
end;


function tKonsole.ErrorCode:word;
begin
  Result:=Kerr;
end; // ErrorCode


function tKonsole.ErrorMsg:string;
begin
  CASE KErr of
    1:      RESULT:='Division durch 0';
    2:      RESULT:='Die Funktion für die übergebenen Parameter nicht definiert';
    3:      RESULT:='Unbekannte Funktion';
    4:      RESULT:='Fehler mit der Verknüpfung (+,-,*,/,^)';
    5:      RESULT:='Eingabefehler bzgl. der Klammersetzung';
    6:      RESULT:='Übergabeparameter zum Berechnen des Funktionswertes nicht geeignet.';
    7:      RESULT:='Das Argument muss eine natürliche Zahl sein';
    8:      RESULT:='Unpassende Anzahl an übergebenen Parametern';
    9:      RESULT:='Eines der Zeichen ist nicht erlaubt';
    10:     RESULT:='Bei der numerischen Integration darf die Funktion nur von einer Variablen abhängen.';
    11:     RESULT:='Numerische Integration: Unbekannte Option.';
    12:     RESULT:='Ungültiger Zahlenbereich';
    13:     RESULT:='Wurzel aus einer negativen Zahl';
    14:     RESULT:='Die Eingabe der Konsole ist leer';
    15:     RESULT:='Argument muss eine Zahl sein';
    16:     RESULT:='Eigabefehler MillerRabinPrim. Es muss gelten: Zahl>0; Zahl mod 2 = 1; Zahl>Zeuge>1';
    17:     RESULT:='Zahlumwandlung ist mißglückt.';
    18:     RESULT:='Dezimalbrüche können mit maximal 16 Stellen Genauigkeit ausgegeben werden.';
    19:     RESULT:='Argument muss eine Gleichung sein';
    20:     RESULT:='Bei Binomialkoeffizienten (N über K) muss N>K>=0 sein.';
  end;
end; //ErrorMsg

end.
